@@ -132,6 +132,7 @@ namespace xcas {
     "end_proc",
     "end_while",
     "et",
+    "except",
     "faire",
     "false",
     "ffaire",
@@ -144,6 +145,7 @@ namespace xcas {
     "fsi",
     "ftantque",
     "function",
+    "global",
     "goto",
     "if",
     "jusqu_a",
@@ -721,7 +723,8 @@ namespace xcas {
     static int program_counter=0;
     Fl_Text_Editor * e = find_editor(m);
     if (e){
-      string tmp,extension;
+      static string tmp;
+      string extension;
       if (Editeur * ed = dynamic_cast<Editeur *>(m->parent()))
 	extension=ed->extension;
       for (;;){
@@ -1882,6 +1885,8 @@ namespace xcas {
   }
 
   void cb_choose_func(Fl_Text_Editor * ed){
+    giac::context * contextptr = get_context(ed);
+    bool python=python_compat(contextptr);
     int dx=240,dy=300,l=14;
     if (ed->window()){
       dx=int(0.5*ed->window()->w());
@@ -1910,7 +1915,7 @@ namespace xcas {
       args->value("x");
       y_ += dh;
       locs=new Fl_Input(dw,y_,dw-2,dh-2,gettext("Locals"));
-      locs->tooltip(gettext("Enter local variable(s), e.g. k or n1,n2,n3. Leave empty if no local variables"));
+      locs->tooltip(gettext("Enter local variable(s), e.g. k or n1,n2,n3. Leave empty if no local variables. Local symbolic variables should be purged after declaration."));
       locs->value("k");
       y_ += dh;
       ret=new Fl_Input(dw,y_,dw-2,dh-2,gettext("Return value"));
@@ -1927,11 +1932,11 @@ namespace xcas {
     w->label((gettext("New function")));
     change_group_fontsize(w,l);
     w->set_modal();
+    if (python) locs->hide(); else locs->show();
     w->show();
     autosave_disabled=true;
     w->hotspot(w);
     Fl::focus(name);
-    context * contextptr=get_context(ed);
     int lang=language(contextptr);
     int r=-2;
     for (;;) {
@@ -1960,8 +1965,6 @@ namespace xcas {
     if (r==0){
       int i=0,addi=0; // i=ed->insert_position(),addi=0;
       string s;
-      giac::context * contextptr = get_context(ed);
-      bool python=python_compat(contextptr);
       switch (xcas_mode(contextptr)){
       case 0:
 	if (python)
@@ -1981,7 +1984,7 @@ namespace xcas {
 	    s+=":={";
 	}
 	s+="\n";
-	if (strlen(locs->value())){
+	if (!python && strlen(locs->value())){
 	  s+=python?"    # local ":"  local ";
 	  s+=locs->value();
 	  if (python) 
@@ -2485,8 +2488,8 @@ namespace xcas {
     {gettext("Test"), 0,  0, 0, 64, 0, 0, 14, 56},
     {gettext("new test"), 0,  (Fl_Callback *) cb_prg_si, 0, 0, 0, 0, 14, 56},
     {gettext("si alors sinon"), 0,  (Fl_Callback *) cb_prg_sialorssinon, 0, 0, 0, 0, 14, 56},
-    {gettext("if [then]"), 0,  (Fl_Callback *) cb_prg_ifthen, 0, 0, 0, 0, 14, 56},
-    {gettext("if [then] else"), 0,  (Fl_Callback *) cb_prg_ifthenelse, 0, 0, 0, 0, 14, 56},
+    {gettext("if"), 0,  (Fl_Callback *) cb_prg_ifthen, 0, 0, 0, 0, 14, 56},
+    {gettext("if else"), 0,  (Fl_Callback *) cb_prg_ifthenelse, 0, 0, 0, 0, 14, 56},
     {gettext("switch"), 0,  (Fl_Callback *) cb_prg_switch, 0, 0, 0, 0, 14, 56},
     {gettext("try catch"), 0,  (Fl_Callback *) cb_prg_trycatch, 0, 0, 0, 0, 14, 56},
     {0}, // end Test
