@@ -68,16 +68,19 @@ namespace giac {
   }
 
   rootmap & symbolic_rootof_list(){
-    static rootmap * ans= new rootmap;
+    static rootmap * ans= 0;
+    if (!ans) ans=new rootmap;
     return *ans;
   }
   rootmap & proot_list(){
-    static rootmap * ans= new rootmap;
+    static rootmap * ans= 0;
+    if (!ans) ans=new rootmap;
     return *ans;
   }
 
   rootmap & galoisconj_list(){
-    static rootmap * ans= new rootmap;
+    static rootmap * ans= 0;
+    if (!ans) ans=new rootmap;
     return *ans;
   }
 
@@ -147,7 +150,7 @@ namespace giac {
     return galoisconj(*g._VECTptr,contextptr);
   }
   static const char _galoisconj_s []="galoisconj";
-  static define_unary_function_eval (__galoisconj,&giac::_galoisconj,_galoisconj_s);
+  static define_unary_function_eval (__galoisconj,&_galoisconj,_galoisconj_s);
   define_unary_function_ptr5( at_galoisconj ,alias_at_galoisconj,&__galoisconj,0,true);
 
   // if true, g is a rootof such that conj(rootof(w))=g
@@ -254,7 +257,7 @@ namespace giac {
 	max_im=cur_im;
       }
       else { // same argument
-	if ( std::abs(cur_re-max_re)<eps*max_re && (cur_im>max_im) ){
+	if ( absdouble(cur_re-max_re)<eps*max_re && (cur_im>max_im) ){
 	  current=*it;
 	  max_im=cur_im;
 	}
@@ -861,10 +864,12 @@ namespace giac {
     }
     vecteur vb(innerdim);
     gen racine_max=undef;
+    bool deep_emb=false; // marker for deep embedding
     if (!trouve){
       // Change for multivariate polynomials p, added evaluation
       if (innerdim){
 	gen params;
+	polynome pb(1),px(unsplitmultivarpoly(p,innerdim));
 	*logptr(contextptr) << gettext("Warning, need to choose a branch for the root of a polynomial with parameters. This might be wrong.") << endl;
 	if (l && l->size()>=2){
 	  for (int i=1;i<l->size();++i){
@@ -875,58 +880,78 @@ namespace giac {
 	  // IMPROVE: using context and *l look for assumptions
 	  if (params.type==_VECT){
 	    vecteur paramv=*params._VECTptr;
-	    for (unsigned j=0;j<paramv.size() && j<vb.size();++j){
-	      gen p=paramv[j];
-	      if (p.type!=_IDNT)
-		continue;
-	      if (p==cst_pi){
-		vb[j]=p;
-		continue;
-	      }
-	      gen g,g2=p._IDNTptr->eval(1,g,contextptr);
-	      if ((g2.type==_VECT) && (g2.subtype==_ASSUME__VECT)){
-		vecteur V=*g2._VECTptr;
-		if ( V.size()==3 && V[1].type==_VECT && V[2].type==_VECT){
-		  for (unsigned i=0;i<V[1]._VECTptr->size();++i){
-		    gen tmp=(*V[1]._VECTptr)[i];
-		    if (tmp.type==_VECT && tmp._VECTptr->size()==2){
-		      gen a=tmp._VECTptr->front(),b=tmp._VECTptr->back();
-		      if (a==minus_inf)
-			vb[j]=b-1;
-		      else {
-			if (b==plus_inf)
-			  vb[j]=a+1;
+	    int nessais=3*paramv.size();
+	    for (int essai=0;essai<nessais;++essai){
+	      for (unsigned j=0;j<paramv.size() && j<vb.size();++j){
+		gen p=paramv[j];
+		if (p.type!=_IDNT)
+		  continue;
+		if (p==cst_pi){
+		  vb[j]=p;
+		  continue;
+		}
+		gen g,g2=p._IDNTptr->eval(1,g,contextptr);
+		if ((g2.type==_VECT) && (g2.subtype==_ASSUME__VECT)){
+		  vecteur V=*g2._VECTptr;
+		  if (V.size()==2)
+		    vb[j]=V[1];
+		  if ( V.size()==3 && V[1].type==_VECT && V[2].type==_VECT){
+		    for (unsigned i=0;i<V[1]._VECTptr->size();++i){
+		      gen tmp=(*V[1]._VECTptr)[i];
+		      if (tmp.type==_VECT && tmp._VECTptr->size()==2){
+			gen a=tmp._VECTptr->front(),b=tmp._VECTptr->back();
+			int decal=1;
+			if (1 || essai)
+			  decal += int((giac_rand(contextptr)*100.0)/rand_max2);
+			if (a==minus_inf)
+			  vb[j]=b-decal;
 			else {
-			  if (a+b==0)
-			    vb[j]=b/2;
-			  else
-			    vb[j]=(a+b)/2;
+			  if (b==plus_inf)
+			    vb[j]=a+decal;
+			  else {
+			    if (a+b==0)
+			      vb[j]=b/(decal+1);
+			    else
+			      vb[j]=(decal*a+b)/(decal+1);
+			  }
 			}
 		      }
 		    }
-		  }
-		} // end if V.size()==3
-	      } // end g2 assume_vect
-	    } // end for j
+		  } // end if V.size()==3
+		} // end g2 assume_vect
+	      } // end for j
+	      vecteur vb0=vb;
+	      find_good_eval(px,pb,vb); 
+	      if (vb0==vb)
+		break;
+	    } // end trying to find a good eval point satisfying assumptions
 	  } // end params.type==_VECT
 	}
 	vecteur vb0=vb;
-	polynome pb(1),px(unsplitmultivarpoly(p,innerdim));
-	find_good_eval(px,pb,vb); // need to modify find_good_eval for assumptions...
+	find_good_eval(px,pb,vb); // find_good_eval does not take care of assumptions, but vb should be ok (loop above)
 	if (vb==vb0)
 	  *logptr(contextptr) << gettext("The choice was done assuming ") << params << "=" << vb << endl;       
 	else 
 	  *logptr(contextptr) << gettext("Non regular value ") << vb0 << gettext(" was discarded and replaced randomly by ") << params << "=" << vb << endl;
-	racines=proot(polynome2poly1(pb));
+	// checking for embedded polynomial coefficients
+	vector< monomial<gen> >::const_iterator it=pb.coord.begin(),itend=pb.coord.end();
+	for (;0 && it!=itend;++it){ // disabled, computations would be too complex
+	  if (it->value.type==_POLY){
+	    deep_emb=true;
+	    break;
+	  }
+	}
+	if (!deep_emb) 
+	  racines=proot(gen2vecteur(evalf(polynome2poly1(pb),1,contextptr)));
       }
       else
-	racines=proot(*b__VECT._VECTptr); 
+	racines=proot(*evalf(b__VECT,1,contextptr)._VECTptr); // evalf to avoid recursion if computing exact roots of b__VECT
       if (is_undef(racines)) return gensizeerr(contextptr);
       // racines= list of approx roots if b__VECT is numeric
       // empty if not numeric
       racine_max=in_select_root(racines,is_real(b__VECT,contextptr),contextptr);
     }
-    if (!trouve && !is_undef(racine_max)){ // select root for b
+    if (!deep_emb && !trouve && !is_undef(racine_max)){ // select root for b
       // now eval each factor over racine_max and choose the one with
       // minimal absolute value
       double min_abs=0;
@@ -1192,7 +1217,7 @@ namespace giac {
   partial_derivative_multiargs D_rootof(&d_rootof);
   */
   static const char _rootof_s []="rootof";
-  static define_unary_function_eval2 (__rootof,&giac::rootof,_rootof_s,&printasrootof);
+  static define_unary_function_eval2 (__rootof,&rootof,_rootof_s,&printasrootof);
   define_unary_function_ptr5( at_rootof ,alias_at_rootof,&__rootof,0,true);
 
   gen max_algext(const gen & args,GIAC_CONTEXT){
@@ -1202,7 +1227,7 @@ namespace giac {
     return MAX_ALG_EXT_ORDER_SIZE=MAX_COMMON_ALG_EXT_ORDER_SIZE=g.val;
   }
   static const char _max_algext_s []="max_algext";
-  static define_unary_function_eval (__max_algext,&giac::max_algext,_max_algext_s);
+  static define_unary_function_eval (__max_algext,&max_algext,_max_algext_s);
   define_unary_function_ptr5( at_max_algext ,alias_at_max_algext,&__max_algext,0,true);
 
   static vecteur sturm(const gen & g){
@@ -1383,7 +1408,7 @@ namespace giac {
     return sturmab(P,x,a,b,contextptr);
   }
   static const char _sturmab_s []="sturmab";
-  static define_unary_function_eval (__sturmab,&giac::_sturmab,_sturmab_s);
+  static define_unary_function_eval (__sturmab,&_sturmab,_sturmab_s);
   define_unary_function_ptr5( at_sturmab ,alias_at_sturmab,&__sturmab,0,true);
 
   gen _sturm(const gen & g,GIAC_CONTEXT){
@@ -1415,7 +1440,7 @@ namespace giac {
     return gendimerr(contextptr);
   }
   static const char _sturm_s []="sturm";
-  static define_unary_function_eval (__sturm,&giac::_sturm,_sturm_s);
+  static define_unary_function_eval (__sturm,&_sturm,_sturm_s);
   define_unary_function_ptr5( at_sturm ,alias_at_sturm,&__sturm,0,true);
 
   gen _sturmseq(const gen & g,GIAC_CONTEXT){
@@ -1424,7 +1449,7 @@ namespace giac {
     return _sturm(g,contextptr);
   }
   static const char _sturmseq_s []="sturmseq";
-  static define_unary_function_eval (__sturmseq,&giac::_sturmseq,_sturmseq_s);
+  static define_unary_function_eval (__sturmseq,&_sturmseq,_sturmseq_s);
   define_unary_function_ptr5( at_sturmseq ,alias_at_sturmseq,&__sturmseq,0,true);
 
   void recompute_minmax(const vecteur & w,const vecteur & range,const gen & expr,const gen & var,gen & resmin,gen & resmax,vecteur & xmin,vecteur & xmax,int direction,GIAC_CONTEXT){
@@ -1642,6 +1667,7 @@ namespace giac {
 	vecteur v=*g2._VECTptr;
 	if ( (v.size()==3) && (v.front()==vecteur(0) || v.front()==_DOUBLE_ || v.front()==_ZINT || v.front()==_SYMB || v.front()==0) && (v[1].type==_VECT)){
 	  a=*v[1]._VECTptr;
+	  if (v.front()==_ZINT) return 3;
 	  return 1;
 	}
 	if (v.size()==1 && v.front()==_ZINT)
