@@ -940,6 +940,7 @@ namespace giac {
       }
       // IMPROVE: if a, b, c are polynomials, search for a polynomial solution
       // of the homogeneous equation, if found we can solve the diffeq
+      gen aa(a),bb(b),cc(c);
       if (lvarxwithinv(makevecteur(a,b,c),x,contextptr)==vecteur(1,x)){
 	vecteur l=vecteur(1,x);
 	gen a0(a),b0(b);
@@ -1020,6 +1021,32 @@ namespace giac {
 	    }
 	  }
 	}
+      } // end polynomial a,b,c
+      a=aa; b=bb; c=cc;
+      if (d==0 && lvarx(makevecteur(a,b,c),x,contextptr)==vecteur(1,x)){
+	// if a,b,c are rationals and d==0, Kovacic
+	gen k=_kovacicsols(makesequence(makevecteur(a,b,c),x),contextptr);
+	if (k.type==_VECT && !k._VECTptr->empty()){
+	  if (k._VECTptr->size()==2){
+	    parameters.push_back(diffeq_constante(int(parameters.size()),contextptr));
+	    parameters.push_back(diffeq_constante(int(parameters.size()),contextptr));	
+	    result=parameters[parameters.size()-2]*k._VECTptr->front()+parameters[parameters.size()-1]*k._VECTptr->back();
+	    return true;
+	  }
+	  if (k._VECTptr->size()==1){
+	    // we have one solution Y, find an independent one as z*Y
+	    gen Y=k._VECTptr->front();
+	    // a*(zY)''+b*(zY)'+c*(zY)=0
+	    // a*(z''Y+2z'Y')+b*(z'Y)=0
+	    // z''*(aY)+z'*(2aY'+bY)=0
+	    result=desolve_lin1(a*Y,2*a*derive(Y,x,contextptr)+b*Y,0,x,parameters,step_info,contextptr);
+	    result=_integrate(makesequence(result,x),contextptr);
+	    parameters.push_back(diffeq_constante(int(parameters.size()),contextptr));
+	    result += parameters.back();
+	    result = result*Y;
+	    return true;
+	  }
+	}
       }
     } // end 2nd order eqdiff
     return false;
@@ -1068,6 +1095,18 @@ namespace giac {
     vx_var=x;
     int save=calc_mode(contextptr);
     calc_mode(0,contextptr);
+#ifdef GIAC_HAS_STO_38
+    // HP Prime: if there is a M0-M9 identifier, this will not work
+    vecteur fid(lidnt(f));
+    for (unsigned i=0;i<fid.size();++i){
+      gen fidi=fid[i];
+      if (fidi.type==_IDNT){
+	const char * ch=fidi._IDNTptr->id_name;
+	if (strlen(ch)==2 && ch[0]=='M' && ch[1]>='0' && ch[1]<='9')
+	  return gensizeerr("Home matrix variable "+ string(ch)+" not allowed. Store your matrix in a CAS variable first");
+      }
+    }
+#endif
     f=remove_equal(eval(f,eval_level(contextptr),contextptr));
     if (ckmatrix(f)){
       vecteur v = *f._VECTptr;
@@ -1754,6 +1793,692 @@ namespace giac {
   static define_unary_function_eval (__Kronecker,&_Kronecker,_Kronecker_s);
   define_unary_function_ptr5( at_Kronecker ,alias_at_Kronecker,&__Kronecker,0,true);
 
+#ifndef USE_GMP_REPLACEMENTS
+  // this code slice (c) 2018 Luka Marohnić 
+  /* returns the coefficient of t in (fraction, Taylor or Laurent) expansion e */
+  gen expansion_coeff(const gen &e,const gen &t,GIAC_CONTEXT) {
+    gen ret(0),g;
+    if (e.is_symb_of_sommet(at_plus) && e._SYMBptr->feuille.type==_VECT) {
+      const vecteur &feu=*e._SYMBptr->feuille._VECTptr;
+      for (const_iterateur it=feu.begin();it!=feu.end();++it) {
+	g=_ratnormal(*it/t,contextptr);
+	if (_evalf(g,contextptr).type==_DOUBLE_) {
+	  ret=g;
+	  break;
+	}
+      }
+    } else {
+      g=_ratnormal(e/t,contextptr);
+      if (_evalf(g,contextptr).type==_DOUBLE_)
+	ret=g;
+    }
+    return ret;
+  }
+
+  bool kovacic_iscase1(const vecteur &poles,int dinf) {
+    if (dinf%2!=0 && dinf<=2)
+      return false;
+    for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+      int order=it->_VECTptr->back().val;
+      if (order%2!=0 && order!=1)
+	return false;
+    }
+    return true;
+  }
+
+  bool kovacic_iscase2(const vecteur &poles) {
+    for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+      int order=it->_VECTptr->back().val;
+      if (order==2 || (order>2 && order%2!=0))
+	return true;
+    }
+    return false;
+  }
+
+  bool kovacic_iscase3(const gen &cpfr,const gen &x,const vecteur &poles,int dinf,GIAC_CONTEXT) {
+    if (dinf<2)
+      return false;
+    vecteur alpha,beta;
+    gen a,b,g(0),p;
+    int d;
+    for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+      p=it->_VECTptr->front();
+      d=it->_VECTptr->back().val;
+      if (d>2)
+	return false;
+      if (d>1) {
+	alpha.push_back(a=expansion_coeff(cpfr,pow(x-p,-2),contextptr));
+	g+=a;
+	a=_eval(sqrt(1+4*a,contextptr),contextptr);
+	if (!_numer(a,contextptr).is_integer() || !_denom(a,contextptr).is_integer())
+	  return false;
+      }
+      beta.push_back(b=expansion_coeff(cpfr,_inv(x-p,contextptr),contextptr));
+      g+=b*p;
+    }
+    g=_eval(sqrt(1+4*g,contextptr),contextptr);
+    return is_zero(_ratnormal(_sum(beta,contextptr),contextptr)) &&
+      _numer(g,contextptr).is_integer() && _denom(g,contextptr).is_integer();
+  }
+
+  void build_E_families(const gen_map &E,const vecteur &cv,vecteur &family,matrice &families) {
+    int i=family.size();
+    if (i>=int(cv.size()))
+      return;
+    const vecteur ev=*E.find(cv[i])->second._VECTptr;
+    for (const_iterateur it=ev.begin();it!=ev.end();++it) {
+      family.push_back(*it);
+      if (family.size()==cv.size())
+	families.push_back(family);
+      else build_E_families(E,cv,family,families);
+      family.pop_back();
+    }
+  }
+
+  void create_identifiers(vecteur &vars,int n) {
+    vars.reserve(n);
+    stringstream ss;
+    for (int i=0;i<n;++i) {
+      ss.str("");
+      ss << " cf" << i;
+      vars.push_back(identificateur(ss.str().c_str()));
+    }
+  }
+
+  gen strip_abs(const gen &g) {
+    if (g.is_symb_of_sommet(at_abs))
+      return g._SYMBptr->feuille;
+    if (g.type==_SYMB) {
+      gen args;
+      if (g._SYMBptr->feuille.type==_VECT) {
+	args=vecteur(0);
+	const vecteur &feu=*g._SYMBptr->feuille._VECTptr;
+	for (const_iterateur it=feu.begin();it!=feu.end();++it) {
+	  args._VECTptr->push_back(strip_abs(*it));
+	}
+      } else args=strip_abs(g._SYMBptr->feuille);
+      return symbolic(g._SYMBptr->sommet,args);
+    }
+    return g;
+  }
+
+  gen explnsimp(const gen &g,GIAC_CONTEXT) {
+    gen e=expand(strip_abs(g),contextptr);
+    e=symbolic(at_exp2pow,symbolic(at_expexpand,symbolic(at_lncollect,e)));
+    return ratnormal(_lin(_eval(e,contextptr),contextptr),contextptr);
+  }
+
+  bool isroot(const gen &g,gen &deg,GIAC_CONTEXT) {
+    if (!g.is_symb_of_sommet(at_pow))
+      return false;
+    const gen &pw=g._SYMBptr->feuille._VECTptr->at(1);
+    return (deg=_inv(pw,contextptr)).is_integer() && !is_minus_one(deg);
+  }
+
+  void partialrad(const gen &g,const gen &deg,gen &outside,gen &inside,bool isdenom,GIAC_CONTEXT) {
+    gen s;
+    if (g.is_integer() && (s=_pow(makesequence(g,_inv(deg,contextptr)),contextptr)).is_integer()) {
+      outside=outside*(isdenom?_inv(s,contextptr):s);
+    } else if (g.is_symb_of_sommet(at_prod) && g._SYMBptr->feuille.type==_VECT) {
+      vecteur &fv=*g._SYMBptr->feuille._VECTptr,qr;
+      for (const_iterateur it=fv.begin();it!=fv.end();++it) {
+	if (it->is_integer()) {
+	  s=_pow(makesequence(*it,_inv(deg,contextptr)),contextptr);
+	  outside=outside*(isdenom?_inv(s,contextptr):s);
+	  continue;
+	} else if (it->is_symb_of_sommet(at_pow)) {
+	  const gen &pw=it->_SYMBptr->feuille._VECTptr->at(1);
+	  const gen &b=it->_SYMBptr->feuille._VECTptr->front();
+	  if (pw.is_integer()) {
+	    qr=*_iquorem(makesequence(pw,deg),contextptr)._VECTptr;
+	    if (isdenom) {
+	      outside=outside/_pow(makesequence(b,qr.front()),contextptr);
+	      inside=inside/_pow(makesequence(b,qr.back()),contextptr);
+	    } else {
+	      outside=outside*_pow(makesequence(b,qr.front()),contextptr);
+	      inside=inside*_pow(makesequence(b,qr.back()),contextptr);
+	    }
+	    continue;
+	  }
+	} else if (it->is_symb_of_sommet(at_inv)) {
+	  partialrad(it->_SYMBptr->feuille,deg,outside,inside,!isdenom,contextptr);
+	  continue;
+	}
+	inside=inside*(isdenom?_inv(*it,contextptr):*it);
+      }
+    } else if (g.is_symb_of_sommet(at_inv))
+      partialrad(g._SYMBptr->feuille,deg,outside,inside,!isdenom,contextptr);
+    else inside=inside*(isdenom?_inv(g,contextptr):g);
+  }
+
+  gen radsimp(const gen &g,GIAC_CONTEXT) {
+    gen deg;
+    if (g.type==_VECT) {
+      vecteur ret;
+      for (const_iterateur it=g._VECTptr->begin();it!=g._VECTptr->end();++it) {
+	ret.push_back(radsimp(*it,contextptr));
+      }
+      return change_subtype(ret,g.subtype);
+    }
+    if (isroot(g,deg,contextptr)) {
+      gen radic=_collect(radsimp(g._SYMBptr->feuille._VECTptr->front(),contextptr),contextptr);
+      gen inside(1),outside(1),inum;
+      partialrad(radic,deg,outside,inside,false,contextptr);
+      gen ideg=_inv(deg,contextptr);
+      inside=_eval(symb_normal(inside),contextptr);
+      if (!(inum=_eval(_pow(makesequence(_numer(inside,contextptr),ideg),contextptr),contextptr)).is_symb_of_sommet(at_pow) ||
+	  inum._SYMBptr->feuille._VECTptr->at(1).is_integer())
+	return _collect(outside,contextptr)*inum/_pow(makesequence(_collect(_denom(inside,contextptr),contextptr),ideg),contextptr);
+      return _collect(outside,contextptr)*_pow(makesequence(_collect(inside,contextptr),ideg),contextptr);
+    }
+    if (g.is_symb_of_sommet(at_prod) && g._SYMBptr->feuille.type==_VECT) {
+      vecteur &feu=*g._SYMBptr->feuille._VECTptr,degv;
+      gen den(1);
+      for (const_iterateur it=feu.begin();it!=feu.end();++it) {
+	if (it->is_symb_of_sommet(at_pow)) {
+	  gen dg=_inv(it->_SYMBptr->feuille._VECTptr->at(1),contextptr);
+	  if (is_greater(dg,2,contextptr))
+	    degv.push_back(dg);
+	} else if (it->is_symb_of_sommet(at_inv))
+	  den=den*it->_SYMBptr->feuille;
+      }
+      if (den.is_symb_of_sommet(at_prod) && den._SYMBptr->feuille.type==_VECT) {
+	vecteur &dfeu=*den._SYMBptr->feuille._VECTptr;
+	for (const_iterateur it=dfeu.begin();it!=dfeu.end();++it) {
+	  if (it->is_symb_of_sommet(at_pow)) {
+	    gen dg=_inv(it->_SYMBptr->feuille._VECTptr->at(1),contextptr);
+	    if (is_greater(dg,2,contextptr))
+	      degv.push_back(dg);
+	  }
+	}
+      }
+      gen gd=_lcm(degv,contextptr);
+      gen p=_collect(ratnormal(pow(g,gd.val),contextptr),contextptr);
+      if (p.is_symb_of_sommet(at_pow)) {
+	gen ppw=p._SYMBptr->feuille._VECTptr->at(1);
+	if (ppw.is_integer())
+	  gd=_eval(gd/ppw,contextptr);
+      }
+      gen ret=_pow(makesequence(p,_inv(gd,contextptr)),contextptr);
+      return isroot(ret,deg,contextptr)?radsimp(ret,contextptr):ratnormal(ret,contextptr);
+    }
+    if (g.type==_SYMB) {
+      gen &feu=g._SYMBptr->feuille;
+      if (feu.type==_VECT) {
+	vecteur res;
+	for (const_iterateur it=feu._VECTptr->begin();it!=feu._VECTptr->end();++it) {
+	  res.push_back(radsimp(*it,contextptr));
+	}
+	return symbolic(g._SYMBptr->sommet,change_subtype(res,feu.subtype));
+      }
+      return symbolic(g._SYMBptr->sommet,radsimp(feu,contextptr));
+    }
+    return g;
+  }
+
+  vecteur strip_gcd(const vecteur &v,GIAC_CONTEXT) {
+    gen g1=_gcd(_apply(makesequence(at_numer,v),contextptr),contextptr);
+    gen g2=_gcd(_apply(makesequence(at_denom,v),contextptr),contextptr);
+    return *_collect(_ratnormal(multvecteur(g2/g1,v),contextptr),contextptr)._VECTptr;
+  }
+
+  gen ratsimp_nonexp(const gen &g,GIAC_CONTEXT) {
+    if (g.type==_VECT) {
+      vecteur res;
+      for (const_iterateur it=g._VECTptr->begin();it!=g._VECTptr->end();++it) {
+	res.push_back(ratsimp_nonexp(*it,contextptr));
+      }
+      return change_subtype(res,g.subtype);
+    }
+    if (g.is_symb_of_sommet(at_plus) && g._SYMBptr->feuille==_VECT) {
+      vecteur &terms=*g._SYMBptr->feuille._VECTptr;
+      gen res(0);
+      for (const_iterateur it=terms.begin();it!=terms.end();++it) {
+	res+=ratsimp_nonexp(*it,contextptr);
+      }
+      return res;
+    }
+    if (g.is_symb_of_sommet(at_prod) && g._SYMBptr->feuille.type==_VECT) {
+      vecteur &facs=*g._SYMBptr->feuille._VECTptr;
+      gen e(1),ne(1);
+      for (const_iterateur it=facs.begin();it!=facs.end();++it) {
+	if (it->is_symb_of_sommet(at_exp))
+	  e=*it*e;
+	else ne=*it*ne;
+      }
+      return _ratnormal(ne,contextptr)*e;
+    }
+    return g;
+  }
+
+  gen fullsimp(const gen &g,GIAC_CONTEXT) {
+    return ratsimp_nonexp(_collect(radsimp(explnsimp(exp(_ratnormal(g,contextptr),contextptr),
+                                                     contextptr),contextptr),contextptr),contextptr);
+  }
+
+  /*
+   * This routine solves the general homogeneous linear second-order ODE
+   * y''=r(t)*y, where r is a non-constant rational function, using Kovacic's
+   * algorithm (https://core.ac.uk/download/pdf/82509765.pdf). A list of
+   * solutions is returned (possibly empty).
+   */
+  gen kovacicsols(const gen &r_orig,const gen &x,const gen &dy_coeff,GIAC_CONTEXT) {
+    gen r=_ratnormal(r_orig,contextptr),inf=symbolic(at_plus,_IDNT_infinity());
+    gen s=_numer(r,contextptr),t=_denom(r,contextptr),a,b,c,e,w=identificateur("omega_");
+    int ds=_degree(makesequence(s,x),contextptr).val;
+    int dt=_degree(makesequence(t,x),contextptr).val,dinf=dt-ds,order,nu;
+    vecteur poles=*_roots(makesequence(t,x),contextptr)._VECTptr,solutions(0);
+    gen cpfr=_cpartfrac(makesequence(r,x),contextptr);
+    gen laur=_series(makesequence(r,x,inf,1,_POLY1__VECT),contextptr);
+    bool success=false;
+    if (kovacic_iscase1(poles,dinf)) {
+      cerr << "Case 1 of Kovacic algorithm" << endl;
+      /* step 1 */
+      gen_map alpha_plus,alpha_minus,sqrt_r;
+      gen alpha_inf_plus,alpha_inf_minus;
+      for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+	c=it->_VECTptr->front();
+	order=it->_VECTptr->back().val;
+	if (order==1) {
+	  sqrt_r[c]=0;
+	  alpha_plus[c]=alpha_minus[c]=1;
+	} else if (order==2) {
+	  sqrt_r[c]=0;
+	  b=expansion_coeff(cpfr,pow(x-c,-2),contextptr);
+	  alpha_plus[c]=(1+sqrt(1+4*b,contextptr))/2;
+	  alpha_minus[c]=(1-sqrt(1+4*b,contextptr))/2;
+	} else if (order%2==0 && order>=4) {
+	  nu=order/2;
+	  e=_series(makesequence(sqrt(r,contextptr),x,c,1,_POLY1__VECT),contextptr);
+	  for (int i=2;i<=nu;++i) {
+	    gen cf=expansion_coeff(e,pow(x-c,-i),contextptr);
+	    if (i==nu)
+	      a=cf;
+	    sqrt_r[c]+=cf/pow(x-c,i);
+	  }
+	  if (is_zero(a))
+	    return false;
+	  b=expansion_coeff(cpfr,pow(x-c,-nu-1),contextptr);
+	  b-=expansion_coeff(_cpartfrac(makesequence(sq(sqrt_r[c]),x),contextptr),pow(x-c,-nu-1),contextptr);
+	  alpha_plus[c]=(nu+b/a)/2;
+	  alpha_minus[c]=(nu-b/a)/2;
+	} else assert(false);
+      }
+      if (dinf>2) {
+	sqrt_r[inf]=0;
+	alpha_inf_plus=0;
+	alpha_inf_minus=1;
+      } else if (dinf==2) {
+	sqrt_r[inf]=0;
+	b=_lcoeff(makesequence(s,x),contextptr)/_lcoeff(makesequence(t,x),contextptr);
+	alpha_inf_plus=(1+sqrt(1+4*b,contextptr))/2;
+	alpha_inf_minus=(1-sqrt(1+4*b,contextptr))/2;
+      } else if (dinf%2==0 && dinf<=0) {
+	nu=-dinf/2;
+	e=_series(makesequence(sqrt(r,contextptr),x,inf,1,_POLY1__VECT),contextptr);
+	for (int i=0;i<=nu;++i) {
+	  gen cf=expansion_coeff(e,pow(x,i),contextptr);
+	  if (i==nu)
+	    a=cf;
+	  sqrt_r[inf]+=cf*pow(x,i);
+	}
+	if (is_zero(a))
+	  return false;
+	b=expansion_coeff(_propfrac(makesequence(r,x),contextptr),pow(x,nu-1),contextptr);
+	b-=expansion_coeff(expand(sq(sqrt_r[inf]),contextptr),pow(x,nu-1),contextptr);
+	alpha_inf_plus=(b/a-nu)/2;
+	alpha_inf_minus=(-b/a-nu)/2;
+      } else assert(false);
+      /* step 2 */
+      int np=poles.size()+1,N=(1<<np),sc,d,maxd=0; // N=std::pow(2.0,np)
+      vecteur fam,cv,vars,v;
+      gen fd,fw,P;
+      for (gen_map::const_iterator it=alpha_plus.begin();it!=alpha_plus.end();++it) {
+	cv.push_back(it->first);
+      }
+      for (int i=0;i<N;++i) {
+	fd=(i & 1)!=0?alpha_inf_plus:alpha_inf_minus;
+	for (int j=1;j<np;++j) {
+	  fd-=(i & (1<<j))!=0?alpha_plus[cv[j-1]]:alpha_minus[cv[j-1]]; // i & (int)std::pow(2.0,j)
+	}
+	fd=_ratnormal(fd,contextptr);
+	if (fd.is_integer() && is_positive(fd,contextptr)) {
+	  fw=(i & 1)!=0?sqrt_r[inf]:-sqrt_r[inf];
+	  for (int j=1;j<np;++j) {
+	    sc=(i & (1<<j)); // i & (int)std::pow(2.0,j)
+	    fw+=sc!=0?sqrt_r[cv[j-1]]:-sqrt_r[cv[j-1]];
+	    fw+=(sc!=0?alpha_plus[cv[j-1]]:alpha_minus[cv[j-1]])/(x-cv[j-1]);
+	  }
+	  fam.push_back(makevecteur(fd,fw));
+	  maxd=std::max(maxd,fd.val);
+	}
+      }
+      /* step 3 */
+      create_identifiers(vars,maxd);
+      for (const_iterateur it=fam.begin();it!=fam.end() && !success;++it) {
+	d=it->_VECTptr->front().val; fw=it->_VECTptr->back();
+	v=vecteur(vars.begin(),vars.begin()+d);
+	P=0;
+	for (int i=0;i<d;++i) P+=v[i]*pow(x,i);
+	P+=pow(x,d);
+	e=_numer(_derive(makesequence(P,x,2),contextptr)+2*fw*_derive(makesequence(P,x),contextptr)+
+		 (_derive(makesequence(fw,x),contextptr)+sq(fw)-r)*P,contextptr);
+	gen lsol=_solve(makesequence(_coeff(makesequence(e,x),contextptr),v),contextptr);
+	if (lsol.type==_VECT && !lsol._VECTptr->empty()) {
+	  lsol=_subst(makesequence(lsol._VECTptr->front(),v,vecteur(d,0)),contextptr);
+	  solutions.push_back(_subst(makesequence(P,v,lsol),contextptr)*
+			      fullsimp(_int(makesequence(fw-dy_coeff/2,x),contextptr),contextptr));
+	  success=true;
+	}
+      }
+    }
+    if (!success && kovacic_iscase2(poles)) {
+      cerr << "Case 2 of Kovacic algorithm" << endl;
+      /* step 1 */
+      gen_map E;
+      for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+	c=it->_VECTptr->front();
+	order=it->_VECTptr->back().val;
+	if (order==1)
+	  E[c]=vecteur(1,4);
+	else if (order==2) {
+	  b=expansion_coeff(cpfr,pow(x-c,-2),contextptr);
+	  E[c]=vecteur(0);
+	  for (int k=-1;k<=1;++k) {
+	    gen tmp=_eval(2+2*k*sqrt(1+4*b,contextptr),contextptr);
+	    if (tmp.is_integer())
+	      E[c]._VECTptr->push_back(tmp);
+	  }
+	} else if (order>2)
+	  E[c]=vecteur(1,order);
+      }
+      vecteur Einf(0);
+      if (dinf>2)
+	Einf=makevecteur(0,2,4);
+      else if (dinf==2) {
+	b=expansion_coeff(laur,pow(x,-2),contextptr);
+	for (int k=-1;k<=1;++k) {
+	  gen tmp=_eval(2+2*k*sqrt(1+4*b,contextptr),contextptr);
+	  if (tmp.is_integer())
+	    Einf.push_back(tmp);
+	}
+      } else if (dinf<2)
+	Einf.push_back(dinf);
+      /* step 2 */
+      vecteur family,families,fam,cv,vars,v;
+      for (gen_map::const_iterator it=E.begin();it!=E.end();++it) {
+	cv.push_back(it->first);
+      }
+      build_E_families(E,cv,family,families);
+      int maxdeg=0,deg;
+      for (const_iterateur it=Einf.begin();it!=Einf.end();++it) {
+	if (families.empty()) {
+	  e=*it/2;
+	  if (e.is_integer() && is_positive(e,contextptr)) {
+	    fam.push_back(makevecteur(e,0));
+	    maxdeg=std::max(maxdeg,e.val);
+	  }
+	}
+	for (const_iterateur jt=families.begin();jt!=families.end();++jt) {
+	  gen th(0);
+	  bool discard=is_one(_even(*it,contextptr));
+	  const vecteur &fm=*(jt->_VECTptr);
+	  for (const_iterateur kt=fm.begin();kt!=fm.end();++kt) {
+	    th+=(*kt)/(x-cv[kt-fm.begin()]);
+	    if (is_zero(_even(*kt,contextptr)))
+	      discard=false;
+	  }
+	  //if (discard) continue;
+	  e=_eval(*it-_sum(fm,contextptr),contextptr)/2;
+	  if (e.is_integer() && is_positive(e,contextptr)) {
+	    fam.push_back(makevecteur(e,th/2));
+	    maxdeg=std::max(maxdeg,e.val);
+	  }
+	}
+      }
+      /* step 3 */
+      create_identifiers(vars,maxdeg);
+      gen P,th,dth;
+      for (const_iterateur it=fam.begin();it!=fam.end() && !success;++it) {
+	deg=it->_VECTptr->front().val; th=it->_VECTptr->back();
+	v=vecteur(vars.begin(),vars.begin()+deg);
+	P=0;
+	for (int i=0;i<deg;++i) P+=v[i]*pow(x,i);
+	P+=pow(x,deg);
+	dth=_derive(makesequence(th,x),contextptr);
+	e=_derive(makesequence(P,x,3),contextptr)+3*th*_derive(makesequence(P,x,2),contextptr)+
+	  (3*sq(th)+3*dth-4*r)*_derive(makesequence(P,x),contextptr)+
+	  (_derive(makesequence(th,x,2),contextptr)+3*th*dth+pow(th,3)-4*r*th-2*_derive(makesequence(r,x),contextptr))*P;
+	e=_numer(e,contextptr);
+	gen cfs=deg==0?undef:_solve(makesequence(_coeff(makesequence(e,x),contextptr),v),contextptr);
+	if (deg==0?is_zero(e):(cfs.type==_VECT && !cfs._VECTptr->empty())) {
+	  if (deg>0) {
+	    cfs=_subst(makesequence(cfs._VECTptr->front(),v,vecteur(deg,0)),contextptr);
+	    P=_subst(makesequence(P,v,cfs),contextptr);
+	  }
+	  gen ph=th+_derive(makesequence(P,x),contextptr)/P;
+	  gen qsol=_solve(makesequence(symb_equal(sq(w)-w*ph+(_derive(makesequence(ph,x),contextptr)/2+sq(ph)/2-r),0),w),contextptr);
+	  if (qsol.type==_VECT) for (const_iterateur jt=qsol._VECTptr->begin();jt!=qsol._VECTptr->end();++jt) {
+	      solutions.push_back(fullsimp(_int(makesequence(*jt-dy_coeff/2,x),contextptr),contextptr));
+	      success=true;
+	    }
+	}
+      }
+    }
+    if (!success && kovacic_iscase3(cpfr,x,poles,dinf,contextptr)) {
+      cerr << "Case 3 of Kovacic algorithm" << endl;
+      vector<int> nv=vecteur_2_vector_int(makevecteur(4,6,12));
+      for (vector<int>::const_iterator nt=nv.begin();nt!=nv.end();++nt) {
+	int n=*nt;
+	/* step 1 */
+	gen_map E;
+	for (const_iterateur it=poles.begin();it!=poles.end();++it) {
+	  c=it->_VECTptr->front();
+	  order=it->_VECTptr->back().val;
+	  if (order==1)
+	    E[c]=vecteur(1,12);
+	  else if (order==2) {
+	    a=expansion_coeff(cpfr,pow(x-c,-2),contextptr);
+	    E[c]=vecteur(0);
+	    for (int k=-n/2;k<=n/2;++k) {
+	      gen tmp=_eval(6+k*(12/n)*sqrt(1+4*a,contextptr),contextptr);
+	      if (tmp.is_integer())
+		E[c]._VECTptr->push_back(tmp);
+	    }
+	  }
+	}
+	vecteur Einf(0);
+	b=dinf>2?gen(0):expansion_coeff(laur,pow(x,-2),contextptr);
+	for (int k=-n/2;k<=n/2;++k) {
+	  gen tmp=_eval(6+k*(12/n)*sqrt(1+4*b,contextptr),contextptr);
+	  if (tmp.is_integer())
+	    Einf.push_back(tmp);
+	}
+	/* step 2 */
+	vecteur family,families,fam,cv,vars,v;
+	for (gen_map::const_iterator it=E.begin();it!=E.end();++it) {
+	  cv.push_back(it->first);
+	}
+	build_E_families(E,cv,family,families);
+	int maxdeg=0,deg;
+	for (const_iterateur it=Einf.begin();it!=Einf.end();++it) {
+	  if (families.empty()) {
+	    e=*it*gen(n)/12;
+	    if (e.is_integer() && is_positive(e,contextptr)) {
+	      fam.push_back(makevecteur(e,0));
+	      maxdeg=std::max(maxdeg,e.val);
+	    }
+	  }
+	  for (const_iterateur jt=families.begin();jt!=families.end();++jt) {
+	    gen th(0);
+	    const vecteur &fm=*(jt->_VECTptr);
+	    for (const_iterateur kt=fm.begin();kt!=fm.end();++kt) {
+	      th+=(*kt)/(x-cv[kt-fm.begin()]);
+	    }
+	    e=gen(n)*_eval(*it-_sum(*jt,contextptr),contextptr)/12;
+	    if (e.is_integer() && is_positive(e,contextptr)) {
+	      fam.push_back(makevecteur(e,gen(n)*th/12));
+	      maxdeg=std::max(maxdeg,e.val);
+	    }
+	  }
+	}
+	/* step 3 */
+	gen S(1);
+	for (const_iterateur it=cv.begin();it!=cv.end();++it) {
+	  S=S*(x-*it);
+	}
+	gen dS=_derive(makesequence(S,x),contextptr),th;
+	vecteur P(n+2,0);
+	create_identifiers(vars,maxdeg);
+	for (const_iterateur it=fam.begin();it!=fam.end();++it) {
+	  deg=it->_VECTptr->front().val; th=it->_VECTptr->back();
+	  v=vecteur(vars.begin(),vars.begin()+deg);
+	  for (int i=0;i<deg;++i) P[n+1]-=v[i]*pow(x,i);
+	  P[n+1]-=pow(x,deg);
+	  P[n]=-S*(_derive(makesequence(P[n+1],x),contextptr)+th*P[n+1]),contextptr;
+	  if (P[n].type==_SYMB) P[n]=_collect(P[n],contextptr);
+	  for (int i=n;i-->0;) {
+	    P[i]=-S*_derive(makesequence(P[i+1],x),contextptr)+((n-i)*dS-S*th)*P[i+1]-(n-i)*(i+1)*sq(S)*r*P[i+2];
+	    if (P[i].type==_SYMB) P[i]=_collect(P[i],contextptr);
+	  }
+	  gen cfs;
+	  if ((deg==0 && is_zero(P[0])) ||
+	      (deg>0 && (cfs=_solve(makesequence(_coeff(makesequence(P[0],x),contextptr),v),contextptr)).type==_VECT &&
+	       !cfs._VECTptr->empty())) {
+	    if (deg>0) {
+	      cfs=_subst(makesequence(cfs._VECTptr->front(),v,vecteur(deg,0)),contextptr);
+	      P=*_subst(makesequence(P,v,cfs),contextptr)._VECTptr;
+	    }
+	    vecteur ac(n+1);
+	    for (int i=0;i<=n;++i) {
+	      ac[i]=_collect(pow(S,i)*P[i+1]/_factorial(n-i,contextptr),contextptr);
+	    }
+	    *logptr(contextptr) << "Warning: outputting the algebraic expression for ω" << endl;
+	    ac=strip_gcd(ac,contextptr);
+	    gen omg=pow(w,4)*ac[4]+pow(w,3)*ac[3]+pow(w,2)*ac[2]+w*ac[1]+ac[0];
+	    if (!is_zero(dy_coeff)) {
+	      vecteur C=*_coeff(makesequence(_subst(makesequence(omg,w,w+dy_coeff/2),contextptr),w),contextptr)._VECTptr;
+	      for (int i=0;i<=n;++i) {
+		ac[i]=_collect(_ratnormal(C[i],contextptr),contextptr);
+	      }
+	      ac=strip_gcd(ac,contextptr);
+	      omg=pow(w,4)*ac[4]+pow(w,3)*ac[3]+pow(w,2)*ac[2]+w*ac[1]+ac[0];
+	    }
+	    return omg;
+	  }
+	}
+      }
+    }
+    return solutions;
+  }
+
+  /*
+   * Return the solution(s) of a second-order linear homogeneous ODE using
+   * Kovacic's algorithm. The first argument is the ODE a(x)*y''+b(x)*y'+c(x)*y=0
+   * itself, which may be given as an expression (left-hand side), an equation or
+   * a list [a,b,c]. The functions a, b and c must be rational in x. The second
+   * and third (both optional) arguments are the independent variable x and the
+   * dependent variable y, respectively. By default, the symbols "x" and "y" are
+   * used.
+   */
+  gen _kovacicsols(const gen &g,GIAC_CONTEXT) {
+    if (g.type==_STRNG && g.subtype==-1) return g;
+    gen x=identificateur("x"),y=identificateur("y"),eq,p(0),q(0),r(0);
+    if (g.type!=_VECT || g.subtype!=_SEQ__VECT) {
+      eq=g;
+    } else if (g.subtype==_SEQ__VECT) {
+      vecteur &gv=*g._VECTptr;
+      if (gv.size()<2) return gensizeerr(contextptr);
+      eq=gv.front();
+      if (gv.size()==2) {
+	if (eq.type==_VECT && gv.back().type==_IDNT)
+	  x=gv.back();
+	else if (gv.back().is_symb_of_sommet(at_of)) {
+	  y=gv.back()._SYMBptr->feuille._VECTptr->front();
+	  x=gv.back()._SYMBptr->feuille._VECTptr->back();
+	} else return gensizeerr(contextptr);
+      } else {
+	if (eq.type==_VECT)
+	  return gensizeerr(contextptr);
+	x=gv[1];
+	y=gv[2];
+      }
+      if (y.type!=_IDNT || x.type!=_IDNT)
+	return gensizeerr(contextptr);
+    }
+    eq=idnteval(eq,contextptr);
+    if (eq.type==_VECT) {
+      vecteur &cfs=*eq._VECTptr;
+      if (cfs.size()!=3)
+	return gensizeerr(contextptr);
+      p=cfs[1]; q=cfs[2]; r=cfs[0];
+    } else if (eq.type==_SYMB) {
+      gen dy=identificateur(" dy"),d2y=identificateur(" d2y"),yx=symb_of(y,x);
+      gen diffy=symbolic(at_derive,y),diff2y=symbolic(at_derive,diffy);
+      eq=_subst(makesequence(eq,makevecteur(_derive(makesequence(yx,x),contextptr),_derive(makesequence(yx,x,2),contextptr)),
+			     makevecteur(dy,d2y)),contextptr);
+      eq=_subst(makesequence(_subst(makesequence(_subst(makesequence(eq,diff2y,d2y),contextptr),diffy,dy),contextptr),yx,y),contextptr);
+      if (eq.is_symb_of_sommet(at_equal))
+	eq=equal2diff(eq);
+      eq=expand(eq,contextptr);
+      vecteur terms=eq.is_symb_of_sommet(at_plus) && eq._SYMBptr->feuille.type==_VECT?
+	*eq._SYMBptr->feuille._VECTptr:vecteur(1,eq);
+      gen tmp;
+      vecteur yvars=makevecteur(y,dy,d2y);
+      for (const_iterateur it=terms.begin();it!=terms.end();++it) {
+	if (is_constant_wrt_vars(tmp=_ratnormal(*it/y,contextptr),yvars,contextptr))
+	  q+=tmp;
+	else if (is_constant_wrt_vars(tmp=_ratnormal(*it/dy,contextptr),yvars,contextptr))
+	  p+=tmp;
+	else if (is_constant_wrt_vars(tmp=_ratnormal(*it/d2y,contextptr),yvars,contextptr))
+	  r+=tmp;
+	else return gensizeerr(contextptr);
+      }
+    } else return gensizeerr(contextptr);
+    if (is_zero(r)) // not a second order ODE
+      return gensizeerr(contextptr);
+    p=_ratnormal(p/r,contextptr);
+    q=_ratnormal(q/r,contextptr);
+    if (rlvarx(p,x).size()+rlvarx(q,x).size()>2) // p or q is not rational in x
+      return gensizeerr(contextptr);
+    /* solve the equation y''+p(x)*y'+q(x)*y=0, transform it first to z''=r(x)*z */
+    r=sq(p)/4+_derive(makesequence(p,x),contextptr)/2-q;
+    return kovacicsols(r,x,p,contextptr);
+  }
+  static const char _kovacicsols_s []="kovacicsols";
+  static define_unary_function_eval (__kovacicsols,&_kovacicsols,_kovacicsols_s);
+  define_unary_function_ptr5(at_kovacicsols,alias_at_kovacicsols,&__kovacicsols,_QUOTE_ARGUMENTS,true)
+
+  /*
+   * Return true iff the expression 'e' is constant with respect to
+   * variables in 'vars'.
+   */
+  bool is_constant_wrt_vars(const gen &e,const vecteur &vars,GIAC_CONTEXT) {
+    for (const_iterateur it=vars.begin();it!=vars.end();++it) {
+      if (!is_constant_wrt(e,*it,contextptr))
+	return false;
+    }
+    return true;
+  }
+  
+  gen idnteval(const gen &g,GIAC_CONTEXT) {
+    if (g.type==_IDNT)
+      return _eval(g,contextptr);
+    if (g.type==_SYMB) {
+      gen &feu=g._SYMBptr->feuille;
+      if (feu.type==_VECT) {
+	vecteur v;
+	for (const_iterateur it=feu._VECTptr->begin();it!=feu._VECTptr->end();++it) {
+	  v.push_back(idnteval(*it,contextptr));
+	}
+	return symbolic(g._SYMBptr->sommet,change_subtype(v,feu.subtype));
+      }
+      return symbolic(g._SYMBptr->sommet,idnteval(feu,contextptr));
+    }
+    return g;
+  }
+
+#endif
 
 #ifndef NO_NAMESPACE_GIAC
 } // namespace giac
