@@ -160,6 +160,7 @@ static int texmacs_counter= 0;
 using namespace giac;
 #define STDIN_FILENO 0
 namespace xcas {
+  /*
   void icas_eval(giac::gen & g,giac::gen & gg,int & reading_file,std::string &filename,giac::context * contextptr){
     gg=protecteval(g,10,contextptr);
   }
@@ -167,6 +168,7 @@ namespace xcas {
   bool fltk_view(const giac::gen & g,giac::gen & ge,const std::string & filename,std::string & figure_filename,int file_type,const giac::context *contextptr){
     return false;
   }
+  */
 }
 
 #else
@@ -377,7 +379,12 @@ string ltgt(const string & s){
   return res;
 }
 
-void verb(std::string & warn,int line,ostream & out,std::string cmd,const std::string & infile,int & texmacs_counter,bool slider,giac::context * contextptr,ostream * checkptr,std::ostream * checkptrin){
+void verb(std::string & warn,int line,ostream & out,std::string cmd_,const std::string & infile,int & texmacs_counter,bool slider,giac::context * contextptr,ostream * checkptr,std::ostream * checkptrin){
+  std::string cmd;
+  if (giac::python_compat(contextptr))
+    cmd="@@"+cmd_;
+  else
+    cmd=cmd_;
   giac::gen g(cmd,contextptr),gg;
   string gs=cmd;
   split(cmd,50);
@@ -467,6 +474,7 @@ void pgiac(std::string infile,std::string outfile,std::ostream * checkptr,std::o
   char buf[BUFFER_SIZE+1];
   buf[BUFFER_SIZE]=0;
   bool inside=false,inprog=false,inverb=false;
+  int inhevea=0; // 1: \ifhevea, -1: \else
   string prg;
   for (;;){
     if (in.eof())
@@ -478,6 +486,27 @@ void pgiac(std::string infile,std::string outfile,std::ostream * checkptr,std::o
     for (;!s.empty();){
       int ss=s.size();
       int pos=0;
+      if (inhevea==1){
+	pos=s.find("\\else");
+	if (pos>=0 && pos<ss){
+	  inhevea=-1;
+	}
+	break;
+      }
+      if (inhevea){
+	pos=s.find("\\fi");
+	if (pos>=0 && pos<ss){
+	  inhevea=0;
+	  break;
+	}
+      }
+      else {
+	pos=s.find("\\ifhevea");
+	if (pos>=0 && pos<ss){
+	  inhevea=1;
+	  break;
+	}
+      }
       if (inside){
 	if (!inverb){
 	  pos=s.find("\\verb");
@@ -547,13 +576,27 @@ void pgiac(std::string infile,std::string outfile,std::ostream * checkptr,std::o
       if (inprog){
 	pos=s.find("\\end{giacprog}");
 	int decal=16;
-	if (pos<0 || pos>=ss)
+	bool hide=false;
+	if (pos<0 || pos>=ss){
 	  pos=s.find("\\end{giaconload}");
+	  if (pos<0 || pos>=ss){
+	    pos=s.find("\\end{giaconloadhide}");
+	    decal=20;
+	    hide=true;
+	  }
+	}
 	else
 	  decal=14;
 	if (pos>=0 && pos<ss){
 	  prg += s.substr(0,pos);
-	  out << "\\begin{verbatim}\n" << prg << "\\end{verbatim}" << endl;
+	  if (hide){
+	    if (prg.substr(prg.size()-3,3)==":;\n")
+	      prg=prg.substr(0,prg.size()-3);
+	  }
+	  else
+	    out << "\\begin{verbatim}\n" << prg << "\\end{verbatim}" << endl;
+ 	  if (giac::python_compat(&ct))
+	    prg="@@"+prg;
 	  giac::gen g(prg,&ct),gg;
 	  int reading_file=0;
 	  std::string filename;
@@ -572,6 +615,15 @@ void pgiac(std::string infile,std::string outfile,std::ostream * checkptr,std::o
 	  if (checkptr) *checkptr << gg << endl;
 	  s=s.substr(pos+decal,ss-decal-pos);
 	  inprog=false;
+	  int graph_output=graph_output_type(gg);
+	  if (graph_output){
+	    filename=giac::remove_extension(infile)+giac::print_INT_(texmacs_counter)+".eps";
+	    string tmp;
+	    if (xcas::fltk_view(g,gg,filename,tmp,-1,&ct)){
+	      out << "\n\\begin{center}\n\\includegraphics[width=0.8\\linewidth]{" << filename << "}\n\\end{center}\n"<<std::endl;
+	      ++texmacs_counter;
+	    }
+	  }
 	  continue;
 	}
 	prg += s + '\n';
@@ -591,11 +643,23 @@ void pgiac(std::string infile,std::string outfile,std::ostream * checkptr,std::o
 	inprog=true;
 	continue;
       }
+      pos=s.find("\\begin{giaconloadhide}");
+      if (pos>=0 && pos<ss){
+	prg = "";
+	s=s.substr(pos+22,ss-22-pos);
+	inprog=true;
+	continue;
+      }
       pos=s.find("\\giac");
       if (inside && pos>=0 && pos<ss){
 	out << s.substr(0,pos) << endl;
 	s=s.substr(pos,ss-pos);
 	ss=s.size();
+	if (s=="\\giacpython" ){
+	  giac::python_compat(1,&ct);
+	  s=s.substr(pos+11,ss-pos-11);
+	  continue;
+	}
 	bool invalid=false;
 	int pos1=s.find("{"),pos2=s.find("}");
 	while ( (pos1>=0 && pos1<ss ) && (pos2<0 || pos2>=ss)){
@@ -948,6 +1012,7 @@ int main(int ARGC, char *ARGV[]){
   }
 #endif
   if (ARGC==2 && (string(ARGV[1])=="-v" || string(ARGV[1])=="--version" ) ){
+    cout << "// (c) 2001, 2018 B. Parisse & others" << endl;
     cout << GIAC_VERSION << endl;
 #ifndef GNUWINCE
     return 0;
@@ -1101,7 +1166,7 @@ int main(int ARGC, char *ARGV[]){
     printf("Giac CAS for mupacs, released under the GPL license 3.0\n");
     printf("See http://www.gnu.org for license details\n");
     printf("May contain BSD licensed software parts (lapack, atlas, tinymt)\n");
-    printf("| (c) 2006, 2017 B. Parisse & al (giac), F.Maltey & al (mupacs) |\n");
+    printf("| (c) 2006, 2018 B. Parisse & al (giac), F.Maltey & al (mupacs) |\n");
     putchar(EMACS_DATA_END);
     bool prompt=true;
     for (int k=0;;++k) {
@@ -1293,7 +1358,7 @@ int main(int ARGC, char *ARGV[]){
     printf("|     Giac CAS for TeXmacs, released under the GPL license (3.0)    |\n");
     printf("|     See http://www.gnu.org for license details                    |\n");
     printf("|  May contain BSD licensed software parts (lapack, atlas, tinymt)  |\n");
-    printf("| (c) 2003,2017 B. Parisse & al (giac), J. van der Hoeven (TeXmacs) |\n");
+    printf("| (c) 2003,2018 B. Parisse & al (giac), J. van der Hoeven (TeXmacs) |\n");
     printf("--------------------------------------------------------------------\n");
     switch (giac::xcas_mode(contextptr)){
     case 0:
@@ -1501,10 +1566,13 @@ int main(int ARGC, char *ARGV[]){
   }
 #ifdef HAVE_LIBREADLINE
   if (ARGC==1){
+    int taillemax=1000;
+    if (getenv("GIAC_TAILLEMAX"))
+      taillemax=atoi(getenv("GIAC_TAILLEMAX"));
     struct tms start, end;  
     using_history();
     cout << "Welcome to giac readline interface" << endl;
-    cout << "(c) 2001,2017 B. Parisse & others" << endl;
+    cout << "(c) 2001,2018 B. Parisse & others" << endl;
     cout << "Homepage http://www-fourier.ujf-grenoble.fr/~parisse/giac.html" << endl;
     cout << "Released under the GPL license 3.0 or above" << endl;
     cout << "See http://www.gnu.org for license details" << endl;
@@ -1552,6 +1620,8 @@ int main(int ARGC, char *ARGV[]){
       giac::messages_to_print="";
 #endif
       giac::gen gq(s,contextptr),ge;
+      if (giac::python_compat(contextptr))
+	gq=giac::equaltosto(gq,contextptr);
       if (giac::first_error_line(contextptr)){
 	cout << parser_error(contextptr);
       }
@@ -1578,10 +1648,7 @@ int main(int ARGC, char *ARGV[]){
 	  cout << "Plot cancelled or unable to plot";
       }
       else {
-	int taillemax=1000;
-	if (getenv("GIAC_TAILLEMAX"))
-	  taillemax=atoi(getenv("GIAC_TAILLEMAX"));
-	string s=taille(ge,taillemax)>taillemax?"Done":ge.print(contextptr);
+	string s=(!insage && taille(ge,taillemax)>taillemax)?"Done":ge.print(contextptr);
 	cout << s;
       }
       cout << endl;
