@@ -579,6 +579,23 @@ namespace giac {
     // check if g may be integrated using the residue formula
     gen A,B,P;
     int typeint=rational?2:is_meromorphic(g,x,A,B,P,contextptr);
+    if (typeint==6 && a==minus_inf && b==plus_inf && is_zero(P) && is_constant_wrt(A,x,contextptr) && lvarxwithinv(B,x,contextptr).size()<2 && is_strictly_positive(-A[0],contextptr)){
+      gen A_(A[0]),B_(A[1]),C_(A[2]);
+      gen der(2*A_*x+B_),tmp(B);
+      B=0;
+      while (!is_zero(tmp,contextptr)){
+	tmp=_quorem(makesequence(tmp,der,x),contextptr);
+	if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
+	  return false;
+	B += tmp._VECTptr->back();
+	tmp=-derive(tmp._VECTptr->front(),x,contextptr);
+      }
+      // int(exp(A_*x^2+B_*x+C_)*B,x,-inf,inf)
+      // =int(exp(A_*(x+B_/2/A_)^2+(-B_^2/4/A_+C_))*B,x,-inf,inf)
+      // =sqrt(pi/A_)*B*exp(B_^2/4/A_-C_)
+      res=sqrt(-cst_pi/A_,contextptr)*B*exp(ratnormal(C_-B_*B_/4/A_,contextptr),contextptr);
+      return true;
+    }
     if (typeint==5){ 
       // A*ln(P(x))+B, A/B/P rational fractions
       bool estreel=is_zero(im(A,contextptr))&&is_zero(im(P,contextptr));
@@ -1118,7 +1135,7 @@ namespace giac {
       return false;
     } // end a==minus_inf
     if (b==plus_inf){
-      gen ga=subst(g,x,x+a,false,contextptr);
+      gen ga_orig=subst(g,x,x+a,false,contextptr),ga(ga_orig);
       // additional check for int(t^n/(exp(alpha*t)-1),t,0,inf)=n!/alpha^(n+1)*Zeta(n+1)
       vecteur vax=rlvarx(ga,x);
       if (vax.size()==2 && vax.front()==x && vax.back().is_symb_of_sommet(at_exp)){
@@ -1138,7 +1155,7 @@ namespace giac {
 		  res=0;
 		  vecteur v=*ganv._VECTptr;
 		  gen facti=pow(expo_a,-2,contextptr);
-		  for (int i=1;i<v.size();++i){
+		  for (int i=1;i<int(v.size());++i){
 		    gen coeff=v[v.size()-i-1];
 		    if (!is_zero(coeff))
 		      res += coeff*facti*Zeta(i+1,contextptr);
@@ -1197,6 +1214,7 @@ namespace giac {
 	  }
 	} // end if (is_linear_wrt(expo...))
       } // end varx.size()==2
+      ga=ga_orig;
       int eo=is_even_odd(ga,x,contextptr);
       if (eo==1){ 
 	vecteur singu=find_singularities(g,*x._IDNTptr,0 /* real singularities*/,contextptr);
