@@ -18,11 +18,14 @@
  */
 
 #include "lpsolve.h"
+#include "optimization.h"
 #include "giac.h"
 #include <ctime>
+
 #ifndef DBL_MAX
 #define DBL_MAX 1.79769313486e+308
 #endif
+
 using namespace std;
 
 #ifndef NO_NAMESPACE_GIAC
@@ -104,9 +107,9 @@ void insert_column(matrice &m,const vecteur &c,int j) {
  */
 void append_column(matrice &m,const vecteur &c) {
     assert(m.size()==c.size());
-    for (int i=0;i<int(m.size());++i) {
-        m[i]._VECTptr->push_back(c[i]);
-    }
+    matrice mt=mtran(m);
+    mt.push_back(c);
+    m=mtran(mt);
 }
 
 /*
@@ -253,7 +256,6 @@ void pivot_ij(matrice &m,int I,int J,bool negate=false) {
  * of Bland's rule to prevent cycling. ith element of 'is_slack' is true iff
  * ith (nonbasic) variable is at its upper bound.
  *
- *
  * If limit>0, simplex algorithm will terminate after that many iterations.
  */
 matrice simplex_reduce_bounded(const matrice &m_orig,const vecteur &u,vector<bool> &is_slack,vecteur &bfs,gen &optimum,
@@ -366,6 +368,7 @@ int lp_node::solve_relaxation() {
     ints cols(ncols),basis;
     vector<bool> is_slack(ncols,false);
     map<int,int> slack_cut;
+    bool is_mip=prob->has_integral_variables();
     //determine upper and lower bounds
     for (int j=0;j<ncols;++j) {
         lp_variable &var=prob->variables[j];
@@ -376,11 +379,11 @@ int lp_node::solve_relaxation() {
             return _LP_INFEASIBLE;
     }
     //populate matrix with consraint coefficients
-    for (int i=0;i<nrows;++i) {
-        row=vecteur(*prob->constr.lhs[i]._VECTptr);
-        m.push_back(row);
+    m=*_matrix(makesequence(nrows,ncols+1,0),prob->ctx)._VECTptr;
+    for (int i=0;i<nrows;++i) for (int j=0;j<ncols;++j) {
+        m[i]._VECTptr->at(j)=prob->constr.lhs[i][j];
     }
-    b=vecteur(prob->constr.rhs);
+    b=prob->constr.rhs;
     //shift variables according to their lower bounds such that l<=x<=u becomes 0<=x'<=u'
     for (int j=0;j<ncols;++j) {
         b=subvecteur(b,multvecteur(l[j],jth_column(m,j)));
@@ -388,7 +391,9 @@ int lp_node::solve_relaxation() {
         obj_ct+=obj[j]*l[j];
         cols[j]=j;
     }
-    append_column(m,b);
+    for (int i=0;i<nrows;++i) {
+        m[i]._VECTptr->at(ncols)=b[i];
+    }
     //assure that the right-hand side column has nonnegative coefficients
     for (iterateur it=m.begin();it!=m.end();++it) {
         if (is_strictly_positive(-it->_VECTptr->back(),prob->ctx)) {
@@ -483,6 +488,7 @@ int lp_node::solve_relaxation() {
         if (is_inf(optimum))
             return _LP_UNBOUNDED; //solution is unbounded
         m.pop_back(); //remove bottom row
+        if (!is_mip) break;
         if (int(cut_indices.size())>=prob->settings.max_cuts)
             break;
         //try to generate Gomory mixed integer cut
@@ -779,6 +785,7 @@ void lp_problem::add_identifiers_from(const gen &g) {
         if (!contains(variable_identifiers,*it))
             variable_identifiers.push_back(*it);
     }
+    variable_identifiers=*_sort(variable_identifiers,ctx)._VECTptr;
 }
 
 /*
@@ -1217,7 +1224,7 @@ glp_prob *lp_problem::glpk_initialize() {
         }
     }
     int n=constr.nrows()*constr.ncols();
-    int *ia=new int[n+1],*ja=new int[n+1],k=0;
+    int *ia=new int[n+1],*ja=new int[n+1]; int k=0;
     double *ar=new double[n+1];
     gen a;
     for (int i=0;i<constr.nrows();++i) {
@@ -1243,9 +1250,9 @@ glp_prob *lp_problem::glpk_initialize() {
         }
     }
     glp_load_matrix(glp,k,ia,ja,ar);
-    delete ia;
-    delete ja;
-    delete ar;
+    delete [] ia;
+    delete [] ja;
+    delete [] ar;
     return glp;
 }
 
@@ -1303,13 +1310,26 @@ int lp_problem::glpk_branchcut(glp_prob *prob) {
     default:
         parm.br_tech=GLP_BR_DTH;
     }
+    switch (settings.nodeselect) {
+    case _LP_DEPTHFIRST:
+        parm.bt_tech=GLP_BT_DFS;
+        break;
+    case _LP_BREADTHFIRST:
+        parm.bt_tech=GLP_BT_BFS;
+        break;
+    case _LP_BEST_PROJECTION:
+        parm.bt_tech=GLP_BT_BPH;
+        break;
+    default:
+        parm.bt_tech=GLP_BT_BLB;
+    }
     return glp_intopt(prob,&parm);
 }
 
 #endif
 
 /*
- * Solve the problem using facilities from GLPK library.
+ * Solve the problem using the GLPK library.
  */
 int lp_problem::glpk_solve() {
 #ifndef HAVE_LIBGLPK
@@ -1482,8 +1502,8 @@ bool lp_problem::glpk_load_from_file(const char *fname) {
                 }
             }
         }
-        delete ind;
-        delete val;
+        delete [] ind;
+        delete [] val;
         stringstream ss;
         const char* name=glp_get_prob_name(prob);
         if (name==NULL)
@@ -1544,7 +1564,9 @@ void parse_limit(const gen &g,int &lim,GIAC_CONTEXT) {
 
 bool parse_options_and_bounds(const_iterateur &it,const_iterateur &itend,lp_problem &prob) {
     for (;it!=itend;++it) {
-        if (it->is_integer()) {
+        if (*it==at_maximize)
+            prob.settings.maximize=true;
+        else if (it->is_integer()) {
             switch(it->val) {
             case _LP_MAXIMIZE:
                 prob.settings.maximize=true;
@@ -1563,6 +1585,8 @@ bool parse_options_and_bounds(const_iterateur &it,const_iterateur &itend,lp_prob
                 if (vi>=0)
                     prob.tighten_variable_bounds(vi,bounds.first,bounds.second);
             }
+            else if (lh==at_maximize && rh.is_integer())
+                prob.settings.maximize=(bool)rh.to_int();
             else if (lh==at_assume && rh.is_integer())
                 prob.settings.assumption=rh.val;
             else if (lh.is_integer()) {
@@ -1737,8 +1761,9 @@ gen _lpsolve(const gen &args,GIAC_CONTEXT) {
     lp_problem prob(contextptr); //create LP problem with default settings
     bool is_matrix_form=false;
     if (it->type==_STRNG) { //problem is given in file
-        const char *fname=gen2string(*it).c_str();
-        if (!prob.glpk_load_from_file(fname))
+        int len=_size(*it,contextptr).val;
+        string fname(it->_STRNGptr->begin(),it->_STRNGptr->begin()+len);
+        if (!prob.glpk_load_from_file(fname.c_str()))
             return undef;
         ++it;
         if (it->type==_VECT)
@@ -1839,7 +1864,7 @@ gen _lpsolve(const gen &args,GIAC_CONTEXT) {
     case _LP_ERROR:
         return undef;
     default: //_LP_SOLVED
-        return _eval(makevecteur(prob.optimum,prob.output_solution()),contextptr);
+        return gen(makevecteur(_eval(prob.optimum,contextptr),_eval(prob.output_solution(),contextptr)),_LIST__VECT);
     }
 }
 static const char _lpsolve_s[]="lpsolve";
