@@ -16,7 +16,7 @@
 
 //#include <fcntl.h>
 #include <cstdlib>
-#ifndef NSPIRE
+#if !defined NSPIRE && !defined FXCG
 #include <cstdio>
 #if defined VISUALC13 && !defined BESTA_OS
 #undef clock
@@ -29,7 +29,7 @@
 #ifdef HAVE_SSTREAM
 #include <sstream>
 #else
-#ifndef NSPIRE
+#if !defined NSPIRE && !defined FXCG
 #include <strstream>
 #endif
 #endif
@@ -39,7 +39,7 @@ using namespace std;
 namespace giac {
 #endif // ndef NO_NAMESPACE_GIAC
 
-#if defined GIAC_HAS_STO_38 || defined NSPIRE
+#if defined GIAC_HAS_STO_38 || defined NSPIRE || defined FXCG
   gen _mathml_error(const gen & g,GIAC_CONTEXT){
     return gensizeerr(gettext("No mathml support"));
   }
@@ -369,7 +369,7 @@ namespace giac {
       if ( 
 	  !need_parenthesis(*itb) || itb->is_symb_of_sommet(at_of)
 	  // (itb->type==_IDNT) || ((itb->type!=_SYMB) && is_positive(*itb,contextptr)) 
-	  || sommetstr=="<mo>=</mo>")
+	  || sommetstr=="<mo>=</mo>" || sommetstr=="<mo>≈</mo>")
 	s=gen2mathml(*itb,contextptr);
       else
 	s="<mo>(</mo>"+gen2mathml(*itb,contextptr)+"<mo>)</mo>";
@@ -378,7 +378,7 @@ namespace giac {
     for (;;){
       if (itb==itend)
 	return s;
-      if ((itb->type!=_SYMB) || sommetstr=="<mo>=</mo>")
+      if ((itb->type!=_SYMB) || sommetstr=="<mo>=</mo>" || sommetstr=="<mo>≈</mo>")
 	s += sommetstr + gen2mathml(*itb,contextptr);
       else
 	s += sommetstr + "<mo>(</mo>"+gen2mathml(*itb,contextptr)+"<mo>)</mo>";
@@ -517,8 +517,44 @@ namespace giac {
 
 
     unary_function_ptr u =mys.sommet;
-    if (u==at_equal || u==at_equal2)
-      return mathml_printsommetasoperator(mys.feuille,"<mo>=</mo>",contextptr);  
+    if (u==at_equal || u==at_equal2){
+      gen chk=eval(mys.feuille[1],1,contextptr);
+      bool eq=true;
+      if (chk.type==_DOUBLE_ || (chk.type==_CPLX && chk.subtype==3))
+	eq=false;
+#if 0 // fails in emscripten
+      if (chk.type==_DOUBLE_){
+	eq=false;
+	double d=chk._DOUBLE_val;
+	if (fabs(d)<=1e5){
+	  d=1e5*d;
+	  if (d==int(d))
+	    eq=true;
+	}
+      }
+      if (chk.type==_CPLX){
+	chk=evalf_double(chk,1,contextptr);
+	eq=false;
+	double d=chk._CPLXptr->_DOUBLE_val;
+	if (fabs(d)<=1e5){
+	  d=1e5*d;
+	  if (d==int(d)){
+	    d=(chk._CPLXptr+1)->_DOUBLE_val;
+	    if (fabs(d)<=1e5){
+	      d=1e5*d;
+	      if (d==int(d)){
+		eq=true;
+	      }
+	    }
+	  }
+	}
+      }
+#endif
+      if (eq)
+	return mathml_printsommetasoperator(mys.feuille,"<mo>=</mo>",contextptr);  
+      else
+	return mathml_printsommetasoperator(mys.feuille,"<mo>≈</mo>",contextptr);
+    }
     if (u==at_different)
       return mathml_printsommetasoperator(mys.feuille,"<mo>≠</mo>",contextptr);
     if (u==at_inferieur_egal)
@@ -572,22 +608,21 @@ namespace giac {
 
   //---------------- Zone SVG  ---------------
 
-  static void svg_dx_dy(double svg_width, double svg_height, double & dx, double & dy){
+  static void svg_dx(double svg_width, double & dx){
     // calibrage des graduations
-    int p_h=(int) std::log10(svg_height);
-    int p_w=(int) std::log10(svg_width);
-    dx=std::pow(10.0,p_w-1);
-    dy=std::pow(10.0,p_h-1);
+    int p_w= int(std::floor( std::log10(svg_width)));
+    dx=std::pow(10.0,p_w); // dx<=svg_width<=10*dx
     if (dx==0)
       dx=1;
-    if (dy==0)
-      dy=1;
-    if (svg_width/dx>25){
-      dx *= 5;
-    }
-    if (svg_height/dy>25){
-      dy *=5;
-    }
+    if (svg_width/dx<=2)
+      dx/=5;
+    while (svg_width/dx<=5)
+      dx/=2;
+  }
+
+  static void svg_dx_dy(double svg_width, double svg_height, double & dx, double & dy){
+    svg_dx(svg_width,dx);
+    svg_dx(svg_height,dy);
   }
 
   void arc_en_ciel(int k,int & r,int & g,int & b){
@@ -636,9 +671,15 @@ namespace giac {
       return "white";
     }
     int r,g,b;
-    if (color>=0x100 && color<0x17e)
+    if (color>=0x100 && color<0x17e){
       color -= 0x100;
-    arc_en_ciel(color,r,g,b);
+      arc_en_ciel(color,r,g,b);
+    } else {
+      r=8*((color>>11)&0x1f);
+      g=4*((color>>5) &0x3f);
+      b=8*(color & 0x1f);
+      //CERR << color << " " << r << " " << g << " " << b << endl;
+    }
     return "rgb("+print_INT_(r)+","+print_INT_(g)+","+print_INT_(b)+")";
   }
 
@@ -679,6 +720,26 @@ namespace giac {
     
   }
 
+  bool axes_off(const gen & g){
+    if (g.type==_VECT){
+      vecteur & v = *g._VECTptr;
+      int vs=int(v.size());
+      for (int i=0;i<vs;++i){
+	if (axes_off(v[i]))
+	  return true;
+	if (v[i].is_symb_of_sommet(at_equal)){
+	  gen f=v[i]._SYMBptr->feuille;
+	  if (f.type==_VECT && f._VECTptr->size()==2){
+	    vecteur w=*f._VECTptr;
+	    if (w.front()==_AXES && w.back()==0)
+	      return true;
+	  }
+	}
+      }
+    }
+    return false;
+  }
+
   // before making a user transformation on the frame, 
   // collect pixon instructions in g
   string svg_preamble_pixel(const gen &g,double svg_width_cm, double svg_height_cm,double xmin,double xmax,double ymin,double ymax,bool ortho,bool xml){
@@ -738,15 +799,17 @@ namespace giac {
     // calibrage des graduations
     double  dx, dy;
     svg_dx_dy(svg_width, svg_height, dx, dy);
-    double i_min_x= xmin/dx;
-    double i_min_y= ymin/dy;
-  
+    double i_min_x= std::ceil(xmin/dx);
+    double i_min_y= std::ceil(ymin/dy);
+    // check axe=0 inside g
+    if (axes_off(g))
+      return buffer;
     //grille  
     double x,y;
     double xthickness((xmax-xmin)/svg_epaisseur1/3),ythickness((ymax-ymin)/svg_epaisseur1/3);
     // double thickness((xmax+ymax-xmin-ymin)/2000);
     
-    for (i=(int) i_min_x; i*dx<xmax; i++){
+    for (i=(int) i_min_x; i*dx<=xmax; i++){
       x=i*dx;
       sprintf(pos,"<line x1=\"%.5g\" y1=\"%.5g\" x2=\"%.5g\" y2=\"%.5g\"",x,ymin,x,ymax);
       pos=buffer+strlen(buffer);
@@ -761,7 +824,7 @@ namespace giac {
       }
       pos=buffer+strlen(buffer);
     }
-    for (i=(int) i_min_y; i*dy<ymax; i++){
+    for (i=(int) i_min_y; i*dy<=ymax; i++){
       y=i*dy;
       sprintf(pos,"<line x1=\"%.5g\" y1=\"%.5g\" x2=\"%.5g\" y2=\"%.5g\"",xmin,y,xmax,y);
       pos=buffer+strlen(buffer);
@@ -778,6 +841,7 @@ namespace giac {
     }
     //cadre
     sprintf(pos,"<rect stroke=\"black\" stroke-width=\"%.5g\" fill=\"none\" x=\"%.5g\" y=\"%.5g\" width=\"%.5g\" height=\"%.5g\" />",2*std::min(xthickness,ythickness),xmin,ymin,svg_width,svg_height);
+    pos=buffer+strlen(buffer);
     // sortie<<"<rect stroke=\""<<"black"<<"\"  stroke-width=\""<<2*thickness<<"\" fill=\"none\" x=\""<<xmin<<"\" y=\""<<ymin<<"\" width=\""<<svg_width<<"\" height=\""<<svg_height<<"\" />"<<endl;  
     // string s=sortie.str();
     return buffer;
@@ -798,8 +862,27 @@ namespace giac {
   string svg_grid(){
     return svg_grid(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax);
   }
+
+  static string greek(const string & s){
+    size_t l=s.size(),deb=0;
+    string cur;
+    for (size_t i=0;i<l;++i){
+      if (!isalpha(s[i])){
+	cur=cur+idnt2mathml_(s.substr(deb,i-deb));
+	deb=i;
+	for (;i<l;++i){
+	  if (isalpha(s[i]))
+	    break;
+	}
+	cur = cur +s.substr(deb,i-deb);
+	deb=i;
+      }
+    }
+    cur=cur+idnt2mathml_(s.substr(deb,l-deb));
+    return cur;
+  }
   
-  string svg_grid(double xmin,double xmax,double ymin,double ymax){
+  string svg_grid(double xmin,double xmax,double ymin,double ymax,const plot_attr & p){
     double svg_width=xmax-xmin;
     double svg_height=ymax-ymin;
 #ifdef HAVE_SSTREAM
@@ -815,50 +898,72 @@ namespace giac {
     double fontscale=x_scale*.6; // (x_scale<y_scale?y_scale:x_scale)*.3;
     char buffer[16384];
     char * pos=buffer;
+#if 0
     sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" height=\"%.5g\" width=\"%.5g\" stroke=\"none\" fill=\"white\"/\n>\n",xmin-x_scale,ymax,y_scale,svg_width+2*x_scale);
     // sortie<<"<rect x=\""<<xmin-x_scale<<"\" y=\""<<ymax<<"\" height=\""<<y_scale<<"\" width=\""<<svg_width+2*x_scale<< "\" stroke=\"none\" fill=\"white\"/>"<<endl;
     pos = buffer+strlen(buffer);
-    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" height=\"%.5g\" width=\"%.5g\" stroke=\"none\" fill=\"white\"/\n>\n",xmin-x_scale,ymin,x_scale,svg_height);
+    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" width=\"%.5g\" height=\"%.5g\" stroke=\"none\" fill=\"white\"/\n>\n",xmin-x_scale,ymin,x_scale,svg_height);
     // sortie<<"<rect x=\""<<xmin-x_scale<<"\" y=\""<<ymin<<"\" width=\""<<x_scale <<"\" height=\""<<svg_height<< "\" stroke=\"none\" fill=\"white\"/>"<<endl;
     pos = buffer+strlen(buffer);
-    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" height=\"%.5g\" width=\"%.5g\" stroke=\"none\" fill=\"white\"/\n>\n",xmax,ymin,x_scale*2,svg_height);
+    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" width=\"%.5g\" height=\"%.5g\" stroke=\"none\" fill=\"white\"/\n>\n",xmax,ymin,x_scale*2,svg_height);
     // sortie<<"<rect x=\""<<xmax<<"\" y=\""<<ymin<<"\" width=\""<<x_scale*2 <<"\" height=\""<<svg_height<<"\" stroke=\"none\" fill=\"white\"/>"<<endl;
     pos = buffer+strlen(buffer);
-    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" height=\"%.5g\" width=\"%.5g\" stroke=\"none\" fill=\"white\"/\n></g>\n",xmin-x_scale,ymin-y_scale,svg_width+2*x_scale,y_scale);
+    sprintf(pos,"<rect x=\"%.5g\" y=\"%.5g\" width=\"%.5g\" height=\"%.5g\" stroke=\"none\" fill=\"white\"/\n></g>\n",xmin-x_scale,ymin-y_scale,svg_width+2*x_scale,y_scale);
     pos = buffer+strlen(buffer);
     // sortie<<"<rect x=\""<<xmin-x_scale<<"\" y=\""<<ymin-y_scale<<"\" width=\""<<svg_width+2*x_scale <<"\" height=\""<<y_scale <<"\" stroke=\"none\" fill=\"white\"/>\n</g>"<<endl;
-
+#else
+    sprintf(pos,"</g>\n");
+    pos = buffer+strlen(buffer);
+#endif
     // calibrage des graduations
     double dx,dy;
     svg_dx_dy(svg_width, svg_height, dx, dy);
     double x,y;
     int i;
-    if ((xmax-xmin)/dx>9)
-      dx*=5;
-    if ((ymax-ymin)/dy>9)
-      dy*=5;
-    double i_min_x= xmin/dx;
-    double i_min_y= ymin/dy;
-    // index des graduations 
+    double i_min_x= std::ceil(xmin/dx);
+    double i_min_y= std::ceil(ymin/dy);
+    bool xleg=p.xlegende.size(),yleg=p.ylegende.size();
+    // For mathml support inside SVG, <foreignObject x="100" y="0" width="250" height="75"> <math xmlns="http://www.w3.org/1998/Math/MathML"> ... </math></foreighObject>
+    // graduations horizontales
+    if (xleg) i_min_x++;
     for (i=(int) i_min_x;  i*dx<=xmax ; ++i){
       x=i*dx;
-      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:middle;\">%.5g</text>\n",x/fontscale,(ymax+0.6*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,1.0,x);
+      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:middle;\">%.4g</text>\n",x/fontscale,(ymax+0.6*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,0.67,x);
       pos = buffer+strlen(buffer);
       // sortie << setprecision(5)<<"<text x=\""<<x<<"\" y=\""<<ymax+0.6*y_scale<<"\" ";
       // sortie <<" style=\"font-size:"<<fontscale<<"pt; text-anchor:middle;\"";
       // sortie << ">" << (float)x <<"</text>"<<endl;
     } 
+    if (xleg){
+      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:start;\">%s</text>\n",xmin/fontscale,(ymax+0.6*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,0.67,greek(p.xlegende).c_str());
+      pos = buffer+strlen(buffer);
+    }
+    // graduations verticales
     for (i=(int) i_min_y;  i*dy<=ymax ; ++i){
       y=i*dy;
-      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:middle;\">%.5g</text>\n",(xmax+0.3*x_scale)/fontscale,(ymax+ymin-y+0.1*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,1.0,y);
+      if (yleg && y<=ymin+0.15*y_scale) continue;
+      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:end;\">%.4g</text>\n",(xmax+0.6*x_scale)/fontscale,(ymax+ymin-y+0.1*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,0.67,y);
       pos = buffer+strlen(buffer);
       // sortie<<setprecision(5)<<"<text x=\""<<xmax+0.2*x_scale<<"\" y=\""<<ymax+ymin-y+0.1*y_scale << "\" ";
       // sortie <<"style=\"font-size:"<<fontscale<<"pt\"";
       // sortie << ">" << (float)y<<"</text>"<<endl;
     }
+    if (yleg){
+      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:end;\">%s</text>\n",(xmax)/fontscale,(ymin+0.0*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,0.67,greek(p.ylegende).c_str());
+      pos = buffer+strlen(buffer);
+    }
+    if (p.title.size()){
+      sprintf(pos,"<text x=\"%.5g\" y=\"%.5g\" transform=\"scale(%.5g,%.5g)\" style=\"font-size:%.5gpt; text-anchor:middle;\">%s</text>\n",(0.5*xmax+0.5*xmin)/fontscale,(ymin+0.0*y_scale)/ratio/fontscale,fontscale,ratio*fontscale,0.67,greek(p.title).c_str());
+      pos = buffer+strlen(buffer);
+    }
     return buffer;
   }
 
+  string svg_grid(double xmin,double xmax,double ymin,double ymax){
+    plot_attr P;
+    return svg_grid(xmin,xmax,ymin,ymax,P);
+  }
+
   static inline string color_string(svg_attribut attr){
     return color_string(attr.color);
   }
@@ -951,6 +1056,14 @@ namespace giac {
       s = s+"\" stroke=\""+color_string(attr)+"\" fill=\"none\" d=\"M";
     g=evalf(g,1,contextptr);
     vecteur v=*(g._VECTptr);
+    for (i=0;i<int(v.size());++i){
+      gen R,I;
+      reim(v[i],R,I,contextptr);
+      if (is_inf(R)||is_inf(I)){
+	v.erase(v.begin()+i);
+	--i;
+      }
+    }
     for (i=0 ; i<signed(v.size()) ; i++){
       s=s+re(v[i],contextptr).print(contextptr)+" "+im(v[i],contextptr).print(contextptr)+" ";
       if (i%2==0)
@@ -964,108 +1077,21 @@ namespace giac {
     if (i%2==1)
       s=s+re(v[i],contextptr).print(contextptr)+" "+im(v[i],contextptr).print(contextptr)+" ";
     s=s+"\" />\n ";
-    // on ??crit la l??gende
-    //recherche d'un point dans le cadre pour ancrer la l??gende
-    for (i=0 ; i<signed(v.size()) ; i++){
-      if (is_positive(re(v[i],contextptr)-xmin,contextptr) 
-	  && is_positive(im(v[i],contextptr)-ymin,contextptr)
-	  && is_positive(xmax-re(v[i],contextptr),contextptr) 
-	  && is_positive(ymax-im(v[i],contextptr),contextptr) ){
-	s = s+svg_text(v[i],legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    //recherche d'un point dans le cadre pour ancrer la legende
+    int vs=int(v.size());
+    for (i=0;i<vs; i++){
+      int j=i<vs/2?vs/2+i:i-vs/2;
+      gen R,I; 
+      reim(v[j],R,I,contextptr);
+      if (is_positive(R-xmin,contextptr) && is_positive(I-ymin,contextptr)
+	  && is_positive(xmax-R,contextptr) && is_positive(ymax-I,contextptr) ){
+	s = s+svg_text(v[j],legende,attr,xmin,xmax,ymin,ymax,contextptr);
 	break;
       }
     }
     return s;
   }
 
-  static string svg_circle(const gen & diameter0, const gen & diameter1, const gen & angle1,const gen & angle2,svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    string s="";
-    gen center=evalf_double(_evalc((diameter0+diameter1)/2,contextptr),1,contextptr);
-    gen centerx,centery;
-    reim(center,centerx,centery,contextptr);
-    double cx=evalf_double(centerx,1,contextptr)._DOUBLE_val;
-    double cy=evalf_double(centery,1,contextptr)._DOUBLE_val;
-    gen d01=evalf_double(_evalc(diameter1-diameter0,contextptr),1,contextptr);
-    gen rotg=arg(d01,0);
-    double rot=evalf_double(rotg,1,contextptr)._DOUBLE_val;
-    gen di,dr;
-    reim(d01,dr,di,contextptr);
-    double did=di._DOUBLE_val,drd=dr._DOUBLE_val;
-    // COUT << center << " " << d01 << " " << drd << " " << did << endl;
-    double r=std::sqrt(drd*drd+did*did)/2.0;
-    // COUT << r << endl;
-    double a1=evalf_double(angle1,1,contextptr)._DOUBLE_val+rot;
-    double a2=evalf_double(angle2,1,contextptr)._DOUBLE_val+rot;
-    if (a2<a1) std::swap(a1,a2);
-    bool arc=std::abs(a2-a1-2*M_PI)>1e-4;
-    if (arc){ // bezier curve is safer...
-      int n=100;
-      vecteur v; v.reserve(n+1);
-      complex<double> c(cx,cy);
-      complex<double> ca=std::exp(complex<double>(0,a1));
-      if (attr.fill_polygon){
-	for (double i=0;i<n/2;++i){
-	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
-	}
-      }
-      for (int i=0;i<=n;++i){
-	double theta=a1+i*(a2-a1)/n;
-	v.push_back(gen(c+r*std::exp(complex<double>(0,theta))));
-      }
-      if (attr.fill_polygon){
-	ca=std::exp(complex<double>(0,a2));
-	for (double i=n/2-1;i>=0;--i){
-	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
-	}
-      }
-      return svg_bezier_curve(v,attr,legende,xmin,xmax,ymin,ymax,contextptr);
-    }
-    s= string(arc?"<path ":"<circle ");
-    if (attr.ie){
-      // COUT << xmin << " " << xmax << " " << ymin << " " << ymax << " " << svg_epaisseur1 << " " << attr.width << endl;
-      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
-      // COUT << thickness << endl;
-      s = s+" stroke-width=\""+print_DOUBLE_(thickness,5)+"\" stroke=\""+color_string(attr);
-    }
-    else
-      s = s+"vector-effect=\"non-scaling-stroke\" stroke=\""+color_string(attr)+"\"  stroke-width=\""+print_INT_(attr.width);
-    // COUT << s << endl;
-    if (attr.fill_polygon)
-      s = s+"\" fill=\""+color_string(attr)+"\" ";
-    else
-      s = s+"\" fill=\"none\" ";
-    if (arc){
-      // arc of circle: compute start.x, start.y, end.x end.y
-      // from polar coord (radius,a1) and (radius,a2)
-      // largeArcFlag=1 if a2-a1>=M_PI
-      // <path d="M start.x start.y 
-      //          A radius radius rotation largeArcFlag 0 end.x end.y
-      //          L center.x center.y Z">
-      // move to start, arc to end, segment to center, loop to start
-      s = s+" d=\"M "+print_DOUBLE_(cx+r*std::cos(a1),5)+" "+print_DOUBLE_(cy+r*std::sin(a1),5);
-#if 1
-      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" 0"+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
-#else
-      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" "+print_DOUBLE_(rot*rad2deg_d,5)+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
-#endif
-      s = s+ print_DOUBLE_(cx+r*std::cos(a2),5)+" "+print_DOUBLE_(cy+r*std::sin(a2),5);
-      if (attr.fill_polygon)
-	s = s+" L "+print_DOUBLE_(cx,5)+" "+print_DOUBLE_(cy,5)+" Z";
-      s = s+"\" />\n";
-      //CERR << s << endl;
-    }
-    else 
-      s = s + "cx=\""+centerx.print(contextptr)+"\" cy=\""
-	+ centery.print(contextptr)+"\" r=\""
-	+ print_DOUBLE_(r,contextptr)
-	+ "\" />\n";
-    if (legende!="")
-      s = s+svg_text(evalf(center+r,1,contextptr),legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    // COUT << s << endl;
-    return s;
-  }
-
-
   static string svg_segment(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
     string s;
     // gen thickness((xmax+ymax-xmin-ymin)/svg_epaisseur1);
@@ -1120,6 +1146,99 @@ namespace giac {
     return s;
   }
 
+  static string svg_circle(const gen & diameter0, const gen & diameter1, const gen & angle1,const gen & angle2,svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string s="";
+    gen center=evalf_double(_evalc((diameter0+diameter1)/2,contextptr),1,contextptr);
+    gen centerx,centery;
+    reim(center,centerx,centery,contextptr);
+    double cx=evalf_double(centerx,1,contextptr)._DOUBLE_val;
+    double cy=evalf_double(centery,1,contextptr)._DOUBLE_val;
+    gen d01=evalf_double(_evalc(diameter1-diameter0,contextptr),1,contextptr);
+    gen rotg=arg(d01,0);
+    double rot=evalf_double(rotg,1,contextptr)._DOUBLE_val;
+    gen di,dr;
+    reim(d01,dr,di,contextptr);
+    double did=di._DOUBLE_val,drd=dr._DOUBLE_val;
+    // COUT << center << " " << d01 << " " << drd << " " << did << endl;
+    double r=std::sqrt(drd*drd+did*did)/2.0;
+    // COUT << r << endl;
+    double a1=evalf_double(angle1,1,contextptr)._DOUBLE_val+rot;
+    double a2=evalf_double(angle2,1,contextptr)._DOUBLE_val+rot;
+    if (a2<a1) std::swap(a1,a2);
+    bool arc=std::abs(a2-a1-2*M_PI)>1e-4;
+    if (arc){ // bezier curve is safer...
+      int n=100;
+      vecteur v; v.reserve(n+1);
+      complex<double> c(cx,cy);
+      complex<double> ca=std::exp(complex<double>(0,a1));
+      if (attr.fill_polygon){
+	for (double i=0;i<n/2;++i){
+	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
+	}
+      }
+      for (int i=0;i<=n;++i){
+	double theta=a1+i*(a2-a1)/n;
+	v.push_back(gen(c+r*std::exp(complex<double>(0,theta))));
+      }
+      if (attr.fill_polygon){
+	ca=std::exp(complex<double>(0,a2));
+	for (double i=n/2-1;i>=0;--i){
+	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
+	}
+      }
+      if (attr.type_line>4){
+	reverse(v.begin(),v.end());
+	s=svg_vecteur(v[v.size()/2],v[v.size()/2+1],attr,"",xmin,xmax,ymin,ymax,contextptr);
+      }
+      return s=s+svg_bezier_curve(v,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+      // return svg_bezier_curve(v,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+    }
+    s= string(arc?"<path ":"<circle ");
+    if (attr.ie){
+      // COUT << xmin << " " << xmax << " " << ymin << " " << ymax << " " << svg_epaisseur1 << " " << attr.width << endl;
+      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
+      // COUT << thickness << endl;
+      s = s+" stroke-width=\""+print_DOUBLE_(thickness,5)+"\" stroke=\""+color_string(attr);
+    }
+    else
+      s = s+"vector-effect=\"non-scaling-stroke\" stroke=\""+color_string(attr)+"\"  stroke-width=\""+print_INT_(attr.width);
+    // COUT << s << endl;
+    if (attr.fill_polygon)
+      s = s+"\" fill=\""+color_string(attr)+"\" ";
+    else
+      s = s+"\" fill=\"none\" ";
+    if (arc){
+      // arc of circle: compute start.x, start.y, end.x end.y
+      // from polar coord (radius,a1) and (radius,a2)
+      // largeArcFlag=1 if a2-a1>=M_PI
+      // <path d="M start.x start.y 
+      //          A radius radius rotation largeArcFlag 0 end.x end.y
+      //          L center.x center.y Z">
+      // move to start, arc to end, segment to center, loop to start
+      s = s+" d=\"M "+print_DOUBLE_(cx+r*std::cos(a1),5)+" "+print_DOUBLE_(cy+r*std::sin(a1),5);
+#if 1
+      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" 0"+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
+#else
+      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" "+print_DOUBLE_(rot*rad2deg_d,5)+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
+#endif
+      s = s+ print_DOUBLE_(cx+r*std::cos(a2),5)+" "+print_DOUBLE_(cy+r*std::sin(a2),5);
+      if (attr.fill_polygon)
+	s = s+" L "+print_DOUBLE_(cx,5)+" "+print_DOUBLE_(cy,5)+" Z";
+      s = s+"\" />\n";
+      //CERR << s << endl;
+    }
+    else 
+      s = s + "cx=\""+centerx.print(contextptr)+"\" cy=\""
+	+ centery.print(contextptr)+"\" r=\""
+	+ print_DOUBLE_(r,contextptr)
+	+ "\" />\n";
+    if (legende!="")
+      s = s+svg_text(evalf(center+r,1,contextptr),legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    // COUT << s << endl;
+    return s;
+  }
+
+
   static string svg_point(gen center, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
     double svg_width=xmax-xmin;
     double svg_height=ymax-ymin;
@@ -1291,7 +1410,7 @@ namespace giac {
 	v[0]=gen(v[0].print(contextptr),contextptr);
       if (v[0].type==_SYMB){ 
 	symbolic figure=*v[0]._SYMBptr; 
-	if (figure.sommet == at_curve){
+	if (figure.sommet == at_curve ){
 	  //CERR << attr.ysurx << endl;
 	  gen curve=figure.feuille[1];
 	  string s;
@@ -1329,6 +1448,8 @@ namespace giac {
 	  }
 	  else
 	    diametre=f;
+	  if (f.type==_VECT && f._VECTptr->size()>=4)
+	    attr.type_line = 5;
 	  if (diametre.type==_VECT && diametre._VECTptr->size()>=2){
 	    vecteur v=*diametre._VECTptr;
 	    return svg_circle(v[0],v[1], angle1,angle2,attr, name,xmin,xmax,ymin,ymax,contextptr);
@@ -1376,11 +1497,25 @@ namespace giac {
   gen _svg(const gen & g,GIAC_CONTEXT){
     if ( g.type==_STRNG && g.subtype==-1) return  g;
     if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>1){
-      if ((*g._VECTptr)[1].type==_STRNG){
-	ofstream of((*g._VECTptr)[1]._STRNGptr->c_str());
-	of << gen2svg(g._VECTptr->front(),contextptr,false) << endl;
+      gen g1=(*g._VECTptr)[1],g0=g._VECTptr->front();
+      if (g1.type==_STRNG){
+	ofstream of(g1._STRNGptr->c_str());
+	of << gen2svg(g0,contextptr,false) << endl;
 	return plus_one;
       }
+      if (g1==1){
+	string s=gen2svg(g0,contextptr,true);
+	plot_attr p;
+	title_legende(g0,p);
+	double window_xmin,window_xmax,window_ymin,window_ymax,window_zmin,window_zmax;
+	vector<double> vx,vy,vz;
+	bool ortho=autoscaleg(g,vx,vy,vz,contextptr);
+	autoscaleminmax(vx,window_xmin,window_xmax,false);
+	autoscaleminmax(vy,window_ymin,window_ymax,false /* fullview */);	
+	bool axes=overwrite_viewbox(g,window_xmin,window_xmax,window_ymin,window_ymax,window_zmin,window_zmax);
+	s = s+ svg_grid(window_xmin,window_xmax,window_ymin,window_ymax,p);
+	return string2gen(s,false);
+      }
       return string2gen(gen2svg(g,contextptr,true),false);
     }
     return string2gen(gen2svg(g,contextptr,false),false);
@@ -1393,58 +1528,81 @@ namespace giac {
 
 
   static string symbolic2mathml(const symbolic & mys, string &svg,GIAC_CONTEXT){
-
     string opstring(mys.sommet.ptr()->print(contextptr));
-    bool of=mys.sommet==at_of;
-    if (of || mys.sommet==at_at) {
-      gen g=mys.feuille[1];
+    const unary_function_ptr & mysommet=mys.sommet;
+    const gen & myfeuille=mys.feuille;
+    bool of=mysommet==at_of;
+    if (of || mysommet==at_at) {
+      gen g=myfeuille[1];
       if (of)
-	return gen2mathml(mys.feuille[0],svg,contextptr)+"<mo>(</mo>"+gen2mathml(g,svg,contextptr)+"<mo>)</mo>";
+	return gen2mathml(myfeuille[0],svg,contextptr)+"<mo>(</mo>"+gen2mathml(g,svg,contextptr)+"<mo>)</mo>";
       g+=array_start(contextptr);
-      return "<msub><mrow>"+gen2mathml(mys.feuille[0],svg,contextptr)+"</mrow><mrow>"+gen2mathml(g,svg,contextptr)+"</mrow></msub>";
+      return "<msub><mrow>"+gen2mathml(myfeuille[0],svg,contextptr)+"</mrow><mrow>"+gen2mathml(g,svg,contextptr)+"</mrow></msub>";
     }
-    if (opstring!="/" && (mys.sommet.ptr()->texprint || mys.sommet==at_different))  
+    if (mysommet==at_program && myfeuille.type==_VECT && myfeuille._VECTptr->size()>=3){
+      const vecteur & v=*myfeuille._VECTptr;
+      gen f=v[2];
+      if (!f.is_symb_of_sommet(at_local)){
+	return gen2mathml(v[0],contextptr)+string("<mo>→</mo>")+gen2mathml(f,contextptr);
+      }
+    }
+    if (opstring!="/" && (mysommet.ptr()->texprint || mysommet==at_different))  
       return mathml_print(mys,contextptr);
-    if (mys.sommet==at_pnt) { 
+    if (mysommet==at_pnt) { 
       svg=svg+symbolic2svg(mys,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,1.0,contextptr);
       return "<mtext>"+mys.print(contextptr)+"</mtext>";
     }
+    if (mysommet==at_member && myfeuille.type==_VECT && myfeuille._VECTptr->size()==2)
+      return gen2mathml(myfeuille._VECTptr->front(),svg,contextptr)+"<mtext>∈</mtext>"+gen2mathml(myfeuille._VECTptr->back(),svg,contextptr);
+    if (mysommet==at_vector){
+      string s="<mover><mrow accent=\"true\">";
+      if (myfeuille.type!=_VECT)
+	s = s+gen2mathml(myfeuille,contextptr);
+      else {
+	const_iterateur it=myfeuille._VECTptr->begin(),itend=myfeuille._VECTptr->end();
+	for (;it!=itend;++it){
+	  s = s + gen2mathml(*it,contextptr);
+	}
+      }
+      s = s+"</mrow><mo mathsize=\"100%\">&rarr;</mo></mover>";
+      return s;
+    }
     gen tmp,value;
-    if (mys.sommet==at_rootof && (tmp=mys.feuille).type==_VECT && tmp._VECTptr->size()==2 && tmp._VECTptr->front().type==_VECT && has_rootof_value(tmp._VECTptr->back(),value,contextptr)){
+    if (mysommet==at_rootof && (tmp=myfeuille).type==_VECT && tmp._VECTptr->size()==2 && tmp._VECTptr->front().type==_VECT && has_rootof_value(tmp._VECTptr->back(),value,contextptr)){
       value=horner_rootof(*tmp._VECTptr->front()._VECTptr,value,contextptr);
       return gen2mathml(value,svg,contextptr);
     }
-    if ( (mys.feuille.type==_VECT) && (mys.feuille._VECTptr->empty()) )
-      return string(provisoire_mbox_begin)+mys.sommet.ptr()->print(contextptr)+string("()")+string(provisoire_mbox_end);
-    if ( (mys.feuille.type!=_VECT) || (mys.feuille._VECTptr->front().type==_VECT)){
-      if (mys.sommet==at_factorial){
-	if (mys.feuille.type==_SYMB)
-	  return "<mrow><mo>(</mo>" + gen2mathml(mys.feuille,contextptr) +"<mo>)</mo></mrow><mtext>!</mtext>";
-	return gen2mathml(mys.feuille,contextptr)+"<mtext>!</mtext>";
+    if ( (myfeuille.type==_VECT) && (myfeuille._VECTptr->empty()) )
+      return string(provisoire_mbox_begin)+mysommet.ptr()->print(contextptr)+string("()")+string(provisoire_mbox_end);
+    if ( (myfeuille.type!=_VECT) || (myfeuille._VECTptr->front().type==_VECT)){
+      if (mysommet==at_factorial){
+	if (myfeuille.type==_SYMB)
+	  return "<mrow><mo>(</mo>" + gen2mathml(myfeuille,contextptr) +"<mo>)</mo></mrow><mtext>!</mtext>";
+	return gen2mathml(myfeuille,contextptr)+"<mtext>!</mtext>";
       }
-      if ((mys.sommet==at_neg) || (mys.sommet==at_plus)){
-	if (mys.feuille.type!=_SYMB) 
-	  return string("<mo>")+mys.sommet.ptr()->print(contextptr)+"</mo>"+gen2mathml(mys.feuille,contextptr); 
-	if (mys.feuille._SYMBptr->sommet==at_inv || mys.feuille._SYMBptr->sommet==at_pow )
-	  return string("<mo>")+mys.sommet.ptr()->print(contextptr)+"</mo>"+gen2mathml(mys.feuille,contextptr) ;
-	return string("<mo>")+mys.sommet.ptr()->print(contextptr)+"</mo>"+string("<mrow><mo>(</mo>") 
-	  + gen2mathml(mys.feuille,contextptr) +string("<mo>)</mo></mrow>");
+      if ((mysommet==at_neg) || (mysommet==at_plus)){
+	if (myfeuille.type!=_SYMB || ( (myfeuille._SYMBptr->sommet==at_at || myfeuille._SYMBptr->sommet==at_of)&& myfeuille._SYMBptr->feuille[0].type==_IDNT) ) 
+	  return string("<mo>")+mysommet.ptr()->print(contextptr)+"</mo>"+gen2mathml(myfeuille,contextptr); 
+	if (myfeuille._SYMBptr->sommet==at_inv || myfeuille._SYMBptr->sommet==at_pow )
+	  return string("<mo>")+mysommet.ptr()->print(contextptr)+"</mo>"+gen2mathml(myfeuille,contextptr) ;
+	return string("<mo>")+mysommet.ptr()->print(contextptr)+"</mo>"+string("<mrow><mo>(</mo>") 
+	  + gen2mathml(myfeuille,contextptr) +string("<mo>)</mo></mrow>");
       }
-      if (mys.sommet==at_inv){
-	return string("<mfrac><mrow><mn>1</mn></mrow><mrow>") + gen2mathml(mys.feuille,contextptr) 
+      if (mysommet==at_inv){
+	return string("<mfrac><mrow><mn>1</mn></mrow><mrow>") + gen2mathml(myfeuille,contextptr) 
 	  + string("</mrow></mfrac>");
       }
-      if (mys.sommet==at_pow) {
-	 return "<msup><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[0],contextptr)+"</mrow><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[1],contextptr)+"</mrow></msup>";
+      if (mysommet==at_pow) {
+	 return "<msup><mrow>"+gen2mathml((*(myfeuille._VECTptr))[0],contextptr)+"</mrow><mrow>"+gen2mathml((*(myfeuille._VECTptr))[1],contextptr)+"</mrow></msup>";
       }
-      return string(provisoire_mbox_begin) +mys.sommet.ptr()->print(contextptr)+ string(provisoire_mbox_end)
-	+ "<mrow><mo>(</mo>" + gen2mathml(mys.feuille,contextptr) +"<mo>)</mo></mrow>" ;
+      return string(provisoire_mbox_begin) +mysommet.ptr()->print(contextptr)+ string(provisoire_mbox_end)
+	+ "<mrow><mo>(</mo>" + gen2mathml(myfeuille,contextptr) +"<mo>)</mo></mrow>" ;
     }
     string s;
-    int l=mys.feuille._VECTptr->size();
-    if ( mys.sommet==at_plus ){
+    int l=myfeuille._VECTptr->size();
+    if ( mysommet==at_plus ){
       for (int i=0;i<l;++i){
-	gen e((*(mys.feuille._VECTptr))[i]);
+	gen e((*(myfeuille._VECTptr))[i]);
 	if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg)){
 	  if ( (e._SYMBptr->feuille).type==_SYMB && (e._SYMBptr->feuille)._SYMBptr->sommet==at_plus)
 	    s = s+string("<mo>-</mo><mrow><mo>(</mo>")
@@ -1466,12 +1624,12 @@ namespace giac {
       } // end_for
       return s;
     }
-    if (mys.sommet==at_prod) {
+    if (mysommet==at_prod) {
       vecteur num;
       vecteur den;
       for (int i=0;i<l;++i){
 
-	gen e((*(mys.feuille._VECTptr))[i]);
+	gen e((*(myfeuille._VECTptr))[i]);
 	if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_inv) )
 	  den.push_back(e._SYMBptr->feuille);
 	else {
@@ -1491,20 +1649,19 @@ namespace giac {
       return s+"<mfrac><mrow>"+prod_vect2mathml_no_bra(num,contextptr)+"</mrow><mrow>"
 	+prod_vect2mathml_no_bra(den,contextptr)+"</mrow></mfrac>";
     } // end if sommet_is_prod
-
-    if (mys.sommet==at_pow){
-      if ( (mys.feuille._VECTptr->back()==plus_one_half)  ){
-    return "<msqrt><mrow>"+gen2mathml(mys.feuille._VECTptr->front(),contextptr)+"</mrow></msqrt>";
+    if (mysommet==at_pow){
+      if ( (myfeuille._VECTptr->back()==plus_one_half)  ){
+    return "<msqrt><mrow>"+gen2mathml(myfeuille._VECTptr->front(),contextptr)+"</mrow></msqrt>";
       }
-      if ( (mys.feuille._VECTptr->back()==minus_one_half ) || 
-	   (mys.feuille._VECTptr->back()==fraction(minus_one,plus_two) ) )
-	return "<mfrac><mn>1</mn><msqrt>"+gen2mathml(mys.feuille._VECTptr->front(),contextptr)+"</msqrt></mfrac>";
-      string s0=gen2mathml((*(mys.feuille._VECTptr))[0],contextptr),s1=gen2mathml((*(mys.feuille._VECTptr))[1],contextptr);
+      if ( (myfeuille._VECTptr->back()==minus_one_half ) || 
+	   (myfeuille._VECTptr->back()==fraction(minus_one,plus_two) ) )
+	return "<mfrac><mn>1</mn><msqrt>"+gen2mathml(myfeuille._VECTptr->front(),contextptr)+"</msqrt></mfrac>";
+      string s0=gen2mathml((*(myfeuille._VECTptr))[0],contextptr),s1=gen2mathml((*(myfeuille._VECTptr))[1],contextptr);
       string s_bra="<msup><mfenced open=\"(\" close=\")\"><mrow>("+s0+")</mrow></mfenced><mrow>"+s1+"</mrow></msup>";
       string s_no_bra= "<msup><mrow> "+s0+"</mrow><mrow>"+s1+"</mrow></msup>";
-      if (mys.feuille._VECTptr->front().type==_SYMB){
+      if (myfeuille._VECTptr->front().type==_SYMB){
 
-	symbolic mantisse(*mys.feuille._VECTptr->front()._SYMBptr);
+	symbolic mantisse(*myfeuille._VECTptr->front()._SYMBptr);
 	if ( (mantisse.feuille.type==_VECT) && (mantisse.feuille._VECTptr->empty()) )
 	  return s_bra;
 	if ( (mantisse.feuille.type!=_VECT) || (mantisse.feuille._VECTptr->front().type==_VECT))
@@ -1514,16 +1671,16 @@ namespace giac {
 	else
 	  return  s_no_bra;
       }
-      else if (mys.feuille._VECTptr->front().type==_FRAC)
+      else if (myfeuille._VECTptr->front().type==_FRAC)
 	return s_bra;
-      else if (mys.feuille._VECTptr->front().type==_CPLX){
-          if  (is_zero(im(mys.feuille._VECTptr->front(),contextptr))) return s_no_bra;
+      else if (myfeuille._VECTptr->front().type==_CPLX){
+          if  (is_zero(im(myfeuille._VECTptr->front(),contextptr))) return s_no_bra;
           else return s_bra;
       }
       //F.H 2*(-1)^n
       //_REAL et _DOUBLE_ inutiles (-1.5)^n passe en exp
-      else if ((mys.feuille._VECTptr->front().type==_INT_)||(mys.feuille._VECTptr->front().type==_ZINT)){
-	if  (is_positive(mys.feuille._VECTptr->front(),contextptr)) return s_no_bra;
+      else if ((myfeuille._VECTptr->front().type==_INT_)||(myfeuille._VECTptr->front().type==_ZINT)){
+	if  (is_positive(myfeuille._VECTptr->front(),contextptr)) return s_no_bra;
           else return s_bra;
       }
       else
@@ -1531,13 +1688,13 @@ namespace giac {
     }
 
     if (opstring=="/") { //at_division non reconnu
-      return "<mfrac><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[0],contextptr)
-	+"</mrow><mrow>"+gen2mathml((*(mys.feuille._VECTptr))[1],contextptr)+"</mrow></mfrac>";
+      return "<mfrac><mrow>"+gen2mathml((*(myfeuille._VECTptr))[0],contextptr)
+	+"</mrow><mrow>"+gen2mathml((*(myfeuille._VECTptr))[1],contextptr)+"</mrow></mfrac>";
     } 
 
     s = string(provisoire_mbox_begin)+opstring + string(provisoire_mbox_end) +"<mrow><mo>(</mo>";
     for (int i=0;;++i){
-      s = s+gen2mathml((*(mys.feuille._VECTptr))[i],contextptr);
+      s = s+gen2mathml((*(myfeuille._VECTptr))[i],contextptr);
       if (i==l-1)
 	return s+"<mo>)</mo></mrow>";
       s = s+',';
@@ -1577,7 +1734,8 @@ namespace giac {
     return "<msub><mi>"+s1+"</mi><mn>"+s2+"</mn></msub>";
   }
 
-  static string idnt2mathml_(const string & sorig){
+  string idnt2mathml_(const string & sorig){
+    if (sorig.empty()) return sorig;
     string s0=sorig;
     int n=int(s0.size()),j;
     for (j=n-1;j>=2;--j){
@@ -1592,9 +1750,23 @@ namespace giac {
       if (s=="mu" || s=="nu" || s=="pi" || s=="xi" || s=="Xi")
 	return "&"+s+';'+sadd;
       if (s=="im")
-	return "&Im"+s+';'+sadd;
+	return "&Im;"+sadd;
       if (s=="re")
-	return "&Re"+sadd;
+	return "&Re;"+sadd;
+      if (s=="RR")
+	return "ℝ"+sadd;
+      if (s=="QQ")
+	return "ℚ"+sadd;
+      if (s=="ZZ")
+	return "ℤ"+sadd;
+      if (s=="CC")
+	return "ℂ"+sadd;
+      if (s=="NN")
+	return "ℕ"+sadd;
+      if (s=="HH")
+	return "ℍ"+sadd;
+      if (s=="PP")
+	return "ℙ"+sadd;
       break;
     case 3:
       if (s=="chi" || s=="phi" || s=="Phi" || s=="eta" || s=="rho" || s=="tau" || s=="psi" || s=="Psi")
@@ -1605,12 +1777,16 @@ namespace giac {
 	return "&"+s+';'+sadd;
       break;
     case 5:
-      if (s=="alpha" || s=="delta" || s=="Delta" || s=="gamma" || s=="Gamma" || s=="kappa" || s=="theta" || s=="Theta" || s=="sigma" || s=="Sigma" || s=="Omega" || s=="omega" || s=="aleph")
-	return "&"+s+';'+sadd;      
+      if (s=="alpha" || s=="delta" || s=="Delta" || s=="gamma" || s=="Gamma" || s=="kappa" || s=="theta" || s=="Theta" || s=="sigma" || s=="Sigma" || s=="Omega" || s=="omega" || s=="aleph" || s=="nabla" 
+	  // || s=="empty" || s=="oplus" || s=="exist" 
+	  )
+	return "&"+s+';'+sadd;
       break;
     case 6:
-      if (s=="lambda" || s=="Lambda" || s=="approx")
-	return "&"+s+';'+sadd;      
+      if (s=="lambda" || s=="Lambda" || s=="approx" 
+	  // || s=="forall" || s=="otimes"
+	  )
+	return "&"+s+';'+sadd;
       break;
     case 7:
       if (s=="epsilon" || s=="product")
@@ -1620,7 +1796,7 @@ namespace giac {
     return s0;
   }
 
-  static string idnt2mathml(const string & s0){
+  string idnt2mathml(const string & s0){
     int n=int(s0.size()),j;
     for (j=n-1;j>=1;--j){
       if (j<n-1 && s0[j]=='_')
